// ================================================================
//  Mega Games - 40+ Games... with a Two level Menu selection
//  for use with mini STEM Educational Game Platform
//  having an 8 LED 4 color bar display and a 4 Button module +Piezo audio out
//  see: https://www.instructables.com/member/RonM9
//  license: Attribution-NonCommercial-ShareAlike

//  Dec 2021 Ronald R. Miller first 40+Mega_Game dev&qa completed
//  14 Jan 2022 Update
//  Added RP2040 support and more games and gadgetry
//  Dec 2023 reorganized (broken into .h files). Now with 50+ activities
//  Updated 25 Sept 2024

// include Library of Hardware Support functions
// and some game& activities
#include "Mini_STEM_Platform.h"
#include "Mega_Games_Utils.h"
#include "SimonSays.h"
#include "Sound_Effects.h"
#include "Experimental.h"
#include "Morse_Code.h"

#include <EEPROM.h>
// the address in the EEPROM we're going to write to
int eeCntAddr = 77;  // totally arbitary

// ====================== local Function prototype predefinitions (good for reference & navigation)
void setup(); // Global initialization
void loop(); // Top Menu level, for selecting an activity
void PushIt(); // Push button to add 1 more to the end a series of flashes, again & again.
void ShootingGallery(); // a Shooting Arcade, Shoot all the targets (lights) to complete a level.
void JumpMan(); // Jump Man - as objects approach jump them or duck under them
void Skee_ball(); // Roll a set of balls into target "hoops" for maximum score
void Sea_Hunt(); // find enemy with Sonar and destroy with Depth Charges
void BiFunc(); // Binary Operator Functions - deduction challenge
void Flip_d(); // One dimensional, "Reversi" game. Reminiscent of 'Go'
void Spray(); // Out spray paint the other guy
void pingPong(); // Ping-Pong - Two player Game
void QuickDraw(); // Ready ... Set ... Draw !
void Tug_a_War(); // Tug (with faster clicks) more than the other to pull in a win
void Chicken(); // Don't bail to win. But if you collide you lose.
void Hot_Hands(); // Recreation of the kid's hand slapping contest
void PIG();  // 'PIG' basketball,  3 misses and you're done
void Joust(); // When your foe is upon you strike him down
void UpDownSide(); // Competitively land balls in indicated spots to dominate
void Whack_Mole(); // see how many Moles you can Whack in 30 seconds
void Speed_Scan_r4(); // hit streaking LED on the right spot to speed things up to a winning max
void KaBoom(); // inspired by Atari 2600 KaBoom! game  https://www.youtube.com/watch?v=_v9D_vDBE40
void Telekinesis(); // LED wanders due to environmental sampling. Challenge your predictive powers.
void PKE_Meter(); // ----- psycho-kinetic scope Detecting Psychokinetic/ParaNormal Energy
int  HRNG(byte algo);  // --- used by PKE_Meter to gather evidence of PKE signals ---
void dispPKE(byte level); // utility to display a representation of the detected PKE levels
void Affinity_Meter(); // implements a Temperament / Affinity / Strength / Love Meter ...
void Fortune_Teller(); // Simulate a Magic-8 ball, rolling of a Die, and a Wheel of Fortune
void playThatTune(); // replay a set of notes played for you.
void React_Now(); // See how fast you a button when the first LED lights up
void SeeIt_CatchIt();  // Capture LED lite states, to get all 4 color LEDs lit or unlit
void Balance(); // keep LED-Ball near the middle, on Simulated tilt-incline of randomly varying magnitude.
void Shuffleboard(); // Inline Shuffleboard. Land a ball in each posible "target" spots to the right.
void Memory(); // There are 4 pairs of items (each a particular sound and illumination), Find the matches.
void CodeBreaker(); // There's a 4 digit entry Btn sequence code. Find it, with Master-Mind like feedback.
void Pairs_Puzzle(); // Flip a combination of pair sets (driven by Btns) to null pattern on display
void Support_Demo(); // Demonstrates use of Hardware support functions, plus Leds and audio level Scales.
void Illumination_Challenge(); // DIY LED Illumination Pattern setup (plus a challenge)
void Automated_Art(); // would-be 'LIFE' simulation in 1 dimension
void tuneLightOrgan();  // Tool to form and evaluate future note sequences and game sounds
void Timer(); //  Timer ... flash & ticks secs and trumpets minutes (+ Stop-Watch functions)
void Metronome(); // Emulates a metronome. 20-240 BPM, Mode selection: 1/4 - 4/4 time.
long readVcc(); // used by readVoltages()
void readVoltages(); // Reports battery voltage (or USB-Vcc volts) plus external ADCs
void btnReporting(); // Button - LED exercise & Reporting
void Color_lights(); // Exercise and test Color operations of . . .
void CursorGlowBeepTest(); // facilitate evaluation of potential Cursor implementations
void DebounceTest(); // user watches for lit LED skipping ahead of button press count
int  readySetGo(int maxDelay); // ------- in support of Head-2-head Games:


void TBD() {
  clearDisp();
  buZZ(25);
  Serial.println("This Selection is Null.");
}
void NA() {TBD();}   // used as a place holder in the middle of a group

/***
   Declare an array of pointers to functions returning void
   and initialize with desired game functions
***/
#ifdef RP2040
void (*games[9][9])() = {
  // 0   ------ Diagnostic Tools & Utilities     NOTE: [0][0] must by TBD
  {TBD, btnReporting, Color_lights, DebounceTest, TBD, TBD, TBD, TBD},
  // 1   ------ Early Inline Games (Oldies & Goodies)
  {setup, PushIt, ShootingGallery, JumpMan, Skee_ball, Sea_Hunt, PIG, BiFunc, Flip_d},
  // 2   ------ Two Person Challenges
  {TBD, pingPong, QuickDraw, Tug_a_War, Chicken, Hot_Hands, Spray, Joust, UpDownSide},
  // 3   ------ Spirted FUN
  {TBD, Whack_Mole, SimonSays, Speed_Scan_r4, KaBoom, Telekinesis, PKE_Meter, Affinity_Meter, Fortune_Teller},   // added KaBoom! for a full set of 8 
  // 4   ------ Show off your Skills
  {TBD, playThatTune, React_Now, SeeIt_CatchIt, Balance, Shuffleboard, Memory, CodeBreaker, Pairs_Puzzle},
  // 5   ------ Demos, Challenges, Activities, and Applets
  {TBD, Support_Demo, Illumination_Challenge, Automated_Art, tuneLightOrgan, SoundEffects, Timer, Metronome, readVoltages},
  // 6   ------ Morse Code Practice & Exercise samples
  {TBD, MC_freeForm, A2Z_Morse_Exercise, Morse_AB_Prompt, MC_Relay_exercise, MC_Receive_Practice, Tx_Line_test, TBD},
  // 7   ------ NEWly Developing
  {TBD, POV_Hello, Rubik, Blinker, SuperComp1960s, allHands, Spin_Puzzle_Lock, TBD},
  // 8
  {TBD, TBD}
};

#else 
void (*games[9][9])() = {   // -----------  used by the Arduino Nano which only has 32KB of program storage
                            // There should be one & only one compiled line per group below !!!
                            // Warning: if you upload code which leaves <200 bytes of dynamic memory you could brick a Nano (so compile before uploading)
  // 0   ------ Diagnostic Tools & Utilities     NOTE: [0][0] must by TBD
  //{TBD, btnReporting, Color_lights, GlowBeepTest, TBD, TBD, TBD, TBD, TBD},
  //{TBD, setup, debugTest, DebounceTest, TBD},  // some debug stuff
  {TBD, setup, TBD},   //  setup is ~ a reset restart (desired for Pro-Micro, to do serial printouts)

  // 1   ------ Early Inline Games (Oldies & Goodies)
  //{TBD, PushIt, ShootingGallery, JumpMan, Skee_ball, Sea_Hunt, PIG, BiFunc, Flip_d},
  {TBD, PushIt, TBD}, //  TBD in cell 1-8 indicate no more valid entries

  // 2   ------ Two Person Challenges
  //{TBD, pingPong, QuickDraw, Tug_a_War, Chicken, Hot_Hands, Spray, Joust, UpDownSide},
  {TBD, pingPong, TBD}, //  TBD in cell 1-8 indicate no more valid entries

  // 3   ------ Spirted FUN
  //{TBD, Whack_Mole, SimonSays, Speed_Scan_r4, KaBoom, Telekinesis, PKE_Meter, Affinity_Meter, Fortune_Teller},  // with KaBoom! for a full set of 8 
  {TBD, Whack_Mole, SimonSays, Speed_Scan_r4, NA, Telekinesis, PKE_Meter, Affinity_Meter, Fortune_Teller},
  //{TBD, TBD}, //  TBD in cell 1-8 indicate no more valid entries

  // 4   ------ Show off your Skills
  //{TBD, playThatTune, React_Now, SeeIt_CatchIt, Balance, Shuffleboard, Memory, CodeBreaker, Pairs_Puzzle},  // now +available: Pairs_Puzzle
  {TBD, playThatTune, React_Now, NA, Balance, Shuffleboard, Memory, CodeBreaker, TBD}, 
  //{TBD, TBD},

  // 5   ------ Demos, Challenges, Activities, and Applets
  //{TBD, Support_Demo, Illumination_Challenge, Automated_Art, tuneLightOrgan, SoundEffects, Timer, Metronome, readVoltages},
  //{TBD, Support_Demo, Illumination_Challenge, Blinker, HW_Blinker, NA, Timer, Metronome, readVoltages}, // alittle smaller for ProMicro
  //{TBD, Support_Demo, NA, SuperComp1960s, Timer, Metronome, TBD}, // alittle smaller for ProMicro
  {TBD, Support_Demo, SuperComp1960s, Metronome, TBD}, // alittle smaller for ProMicro
  //{TBD, TBD},

  // 6   ------ Morse Code Practice & Exercise samples
  ////{TBD, MC_freeForm, A2Z_Morse_Exercise, Morse_AB_Prompt, MC_Relay_exercise, MC_Receive_Practice, Tx_Line_test, TBD},
  // This group alone does NOT fit in an Nano.  For use in sketches 'Morse_Training' or 'Morse_Practice' on a Arduino Nano as well
  //{TBD, Morse_AB_Prompt, TBD},  // one here, at a time only please, and you have to minimize some of the above groups
  {TBD, TBD},

  // 7   ------ Newly Developing
  //{TBD, POV_Hello, Rubik, Blinker, SuperComp60s, allHands, Spin_Puzzle_Lock, TBD},
  {TBD, TBD},

  // 8
  {TBD, TBD}
};
#endif

byte gameSets; // = whats filled in above (# of valid rows, without 'TDB' in cell [1])
byte entries[9];
unsigned int idleCnt = 0, loopCnt;

/*** Code herein No longer relys on 1 milisecond update routine for input (btns) of output (LED) Updates
 *   But instead directly manages the Scanning of input btns and Refreshing of the LED "screen"  
 *   Further without the ISR I am sure to keep the main code capable of working smooth with or without it
 ***/

// the setup routine runs once on power up or when you press reset:
//=================================================================
void setup() { // Global initialization
  int aval;
  unsigned long gsum;  // keep gsum as uninitialized

  // initialize serial communication at 9600 bits per second:
  Serial.begin(115200);
  delay(1000);  // else first prints get lost
  Serial.println("Mega Games 50+ utilizing the mini LED STEM Platform");

 #ifdef RP2040
  // init EEPROM emulation on RP2040
  EEPROM.begin(1024);  // same size as a Nano has
 #endif

  // determine the # of valid entries in games[][]
  for (int i = 0; i <= nleds; i++) {
    entries[i] = nleds; // herein a max of 8 entries
    for (int j = 1; j <= nleds; j++) {
      if (games[i][j] == games[0][0]) {
        entries[i] = j - 1; // set not fill to the limit
        break;
      }
    }
    //Serial.print(i); Serial.print("  entries: "); Serial.println(entries[i]);
    if (entries[i] > 0) gameSets = i + 1;
  }
  Serial.print("  groups: "); Serial.println(gameSets);

  // ------------ setup the Mini Game/STEM platform (8LEDs, 4Btns, and Audio)
  gPlatform_setup();
  if (gameSets > 6) MC_setup(); // setup for Morse Code operations

  beep(50, 50); beep(50, 50); beep(150); // announce "it is alive!"

  // accumulate pseudo random values, due to HW input noise (Entropy), then randomize
  // used when random values need to be generated prior to (/without) ramdomly occuring user input
  //asum=dsum=0;
  t0 = micros();
  for (int i = 1; i <= 100; i++) { // work up a random seed value
    aval = analogRead(ADC_b) + analogRead(ADC_a) + 1;
    gsum = gsum + i * aval;
    delayMicroseconds(500 + gsum % 1000); // average 1000 us
  }
  randSeed = (gsum + micros()) % 9999999; // here you could set a seed to a given value for repeated testing
  Serial.print("Random Seed used: "); Serial.println(randSeed);
  randomSeed(randSeed); //Added to generate "fresh randomness" with the randomArray for the output function
}

// the loop routine runs over and over again forever:
//=================================================================
void loop() { // Top Menu level, for selecting an activity
  byte func, maxFunc;

  //group = 1; // set at reset
  if (1>group || group>8) group = 1;
  func = 1;

  group = get_Option(gameSets - 1, group); // select Group of interest   (can adj volume with Btn4 while here)
  if (entries[group] == 0) {
    buZZ(75);
    Serial.print("SORRY, nothing in that group.");
    clearDisp();
    delay(700);
    return;
  }
  Serial.print(" Group: "); Serial.println(group);

  while (1) {
    pAdj = true;  // enable MC ditdah period adjustment
    prtChr = true; // MC chr prints as fetched
    debounceDelay = DEBOUNCE; // re-establich STD setting

    glowEnable = false;
    clearDisp();
    while (Btn) {   // wait for no button
      if (btn4) buZZ(1);
      refreshWait(1);
      scanBtns();
    }

    // currently there are upto 8 activities/games supported
    maxFunc = entries[group];
    // indicate selectable functions
    for (int i = 1; i <= maxFunc; i++) {
      dim[i] = true;
    }
    brightOne = func;
    refreshDisp();

    //    while(Btn) {scanBtns(); refreshDisp();}  // wait here for button release
    while (! btn1) {
      refreshWait(2); // 2ms so are selection is extra sure footed, incase of bouncy switches
      scanBtns();
      //if (btn2Pressed) brightOne = (brightOne==1)?  maxFunc : brightOne-1;  // -- move left
      if (btn2Pressed) brightOne = (brightOne == 0) ?  maxFunc : brightOne-1; // -- move left Allow 0:debugging
      if (btn3Pressed) brightOne = (brightOne % maxFunc) + 1; // ++ moves right
      if (games[group][brightOne] == TBD) brightOne=maxFunc;  // skip is no func 0
      if (ESC || btn4) {
        boop(50);
        return;
      }
    }
    if (randSeed == 0) {
      randSeed = random(micros() / 4); // Add "more randomness" to random() calls, based on time btn pressed
      randomSeed(randSeed);       // for repeated testing, set randSeed=desired initial value
      Serial.print("random Seed:  "); Serial.println( randSeed);
    }
    gameID = random(999999999);

    func = brightOne;
    clearDisp();
    level = 0;
    while (Btn) scanBtns(); // wait for button release

    Serial.print(" FUNC: "); Serial.println(func);

    games[group][func]();   // ------------------ Call the Selected Function
    if (func == 0) func = 1;
    if (group == 0) return; // start all over if any diag type code was run

    pinMode(ADC_a, INPUT);  // be sure to have it reinstated without PullUp
    pinMode(ADC_b, INPUT);  // be sure to have it reinstated without PullUp
    pinMode(SENSOR, INPUT);  // be sure to have it reinstated without PullUp
    digitalWrite(AUDIO_NEG, LOW); // will have no effect if it is set as INPUT
 #ifdef RP2040
 #ifdef DIO
    pinMode(DIO, OUTPUT);   // provide a GRN reference for use of adjasent pins A2-D0 on Ext conn 
    digitalWrite(DIO, LOW);
 #endif
 #endif

  }
}

// ========================   Game Loops   =============================

// ************************  Group 1  *******************************
// Various selection of mini Apps / Games / Challenges
//=============================================================
// from Menu16Games Set #1 (reworked for 8 instead of 12 LEDs & made dependant on 'nleds')  

void PushIt() { // Push button to add 1 more to the end a series of flashes, again & again.
  // Nov 2017 reworked 'PushIt' operation to be useful with displays of < 12 Leds
  bool lastPressed, pressed = false, pushed = false;

  static int maxCnt = 2; // starts at 2 after power on
  int litPeriod, period, cnt, lastMaxCnt, savedCnt;
  bool isLit, incDone = false;

  clearDisp();
  refreshDisp();

  // Push-It init code
  // --------------------------
  litPeriod = 1000;
  period = litPeriod;
  cnt = 0; // =1 resulted in initially one too few flashes

  // read previous incarnation of 'max count' from EEPROM for the 'Push-It' game
  savedCnt = EEPROM.read(eeCntAddr);
  maxCnt = (savedCnt < 50)? savedCnt: 2; // be reasonable
  showLevel(maxCnt);

  digitalWrite(ledOnBrd, HIGH);  //needs testing with RP2040
  t0 = millis();
  isLit = false;

  while (1) {
    pressed = btnPressed() && !btn4;
    if (!lastPressed && pressed)  {
      pushed = true;
    }

    // Flash from 1 to n times
    // Increasing the number every time the user has pressed the button on the first flash of the series
    // A missed attempt will reduce the count

    if ((millis() - t0) >= period) {  //    is this time period up ?
      if (cnt <= 1) {
        litPeriod = (nleds < 12) ? 800 : 1000;
        for (int i = 1; i < maxCnt; i++) {
          //good for 12 leds:    litPeriod = 16 * (litPeriod-100) / 20 +100;  //  P = 0.8*(P-100) + 100
          litPeriod = 15 * (litPeriod - 80) / 20 + 80; //  P = 0.75*(P-80) + 80
        }
      }

      if (isLit) {                       // about to go from LED on to OFF
        if (cnt == maxCnt + 1) { // on orig 'Push-It' this was if (cnt == 1) {
          //             !!!!!!!!!!!!!!!! ------------------------------------  time to judge
          lastMaxCnt = maxCnt;
          if (pushed) {       // he hit the mark
            maxCnt++;
            cell[lastMaxCnt] = 1;
            //            litLED=maxCnt;    // acknowledge with a new bright LED
            showLevel(maxCnt);  // now ack with 'showLevel' so we can discern levels above nleds
            refreshWait(500);
            incDone = true;
            if (maxCnt > nleds) {
              refreshWait(500);
              taDah();       // reached the highest level
              //              maxCnt=2;   // now, allow levels > nleds
            }
          }

          if (maxCnt != lastMaxCnt) {
            Serial.print("maxCnt = ");
            Serial.println(maxCnt);
          }
          if (savedCnt != maxCnt && (maxCnt == lastMaxCnt)) {  // dont write more often than needed
            savedCnt = maxCnt;
            EEPROM.write(eeCntAddr, maxCnt);
          }
          cnt = maxCnt + 1;
        } else {
          // isLit and not time to judge
        }
        period = litPeriod / 3;

      } else {                     // about to go from LED off to ON
        period = litPeriod;
        cnt++;
        if (cnt > (maxCnt + 1)) { // at maxCnt LED is off for one count
          cnt = 1;             // ===========  restart the cycle
          incDone = false;
          digitalWrite(ledOnBrd, HIGH); // needs testing with RP2040
        }
        //      Serial.print(cnt);
      }
      t0 = millis();
      isLit = not isLit;

      if (cnt > 0 && cnt <= maxCnt) {  // && cnt < maxCnt
        if (pushed && !incDone) {
          maxCnt--;
          if (maxCnt < 1) maxCnt = 1;
          cell[maxCnt] = 0;
          brightOne = 0;
          beep(100);    // had a miss & a strike
        }
      }
      lastPressed = pressed;
      pushed = false;
    }

    //-------------- UPDATE Display
    litLED = (maxCnt - 1) % nleds + 1;
    if (litLED == 1) litLED = 0; // so you can tell when it flashes
    for (int i = 1; i <= nleds; i++) {
        dim[i] = (i <= maxCnt);
    }
    if (maxCnt > nleds) { // make it easier to see the fast flashes
      brightOne = 0;
      spriteTwo = 0;
      dim[1] = 0;
    }

    if (isLit && cnt < maxCnt + 1) spriteTwo = 1; // brighten led #1
    else spriteTwo = nleds + 1; //ledON(13); // No 'ON' lit but used up one's normal time, so dim leds don't flicker  ###TODO
    if (cnt == maxCnt + 1 && pushed) {
      spriteTwo = (maxCnt + 1);
    }
    if (spriteOne > 0) {
      refreshWait(500);
      spriteOne = 0;
    }

    refreshWait(1);
    if (ESC || btn4) {
      refreshWait(300);
 #ifdef RP2040
      EEPROM.commit();  // use with RP2040
 #endif
      return;
    }

    if (btn2) { // re initial game level
      maxCnt = 2;
      brightOne = 0;
      cnt = 0;
    }
  }
}

// =====================================================================
void ShootingGallery() { // a Shooting Arcade, Shoot all the targets (lights) to complete a level.
  int targs;
  bool playing = true;
  int cycleTime;
  int scanCell, targCell;
 
  byte ver = getOption(2);  // version 1: sight moves & targets are stationary
  // version 2: sight stationary & targets move
  // ver 3&4 the targets can only be seen when under the sight, but is too difficult for me, so commented out below
  if (ver == 0) return;

  level = 1;
  while (1) {
    if (level > 10) level = 1;  // setup game-level
    clearDisp();
    for (int i = 1; i <= nleds; i++) cell[i] = 0;
    
    for (int i=0; i<(nleds/2+1); i++) {cell[PRIG(i, gameID, nleds)+1]=1;} // load up some targets
    gameID++;

    while (btnPressed()) {
      refreshWait(1); // wait for no button pressed
    }

    playing = true;
    while (playing) {
      scanCell = scanCell % nleds + 1;   // goes from 1 to nleds
      targs = 0;
      for (int i = 1; i <= nleds; i++) {
        if (cell[i]) targs++;
      }
      if (targs == 0) {
        //horray();
        taDah();
        level++;
        //playing=false; // restart game at next level
        break;
      }

      //================ UPDATE Display

      if (ver == 1) {
        for (int i = 1; i <= nleds; i++) {
          if (ver <= 2) dim[i] = cell[i];
        }
        litLED = scanCell;
      } else if (ver == 2) { // ver is 2
        for (int i = 1; i <= nleds; i++) {
          dim[i] = cell[(scanCell + i) % nleds + 1]; // causes a moving shifted display of the cells
        }
        if (dim[nleds / 2]) brightOne = nleds / 2;
        else brightOne = 0;
        dim[nleds / 2] = true;
      }
      /***
            // you have to shoot a few blindly, then be very keen as to when there is a target in your sights (when it goes bright)
            if (ver==3) {
              for (i=1; i<=nleds; i++) { dim[i] = false; }
              dim[scanCell]=true; // comment out for high contrast (off/bright)
              if (cell[scanCell]) brightOne = scanCell;
              else brightOne=0;
            } else if (ver==4){
              targCell = (scanCell+6)%nleds+1;
              if (cell[targCell]) brightOne = 6;
              else brightOne=0;
              dim[6]=true;
            }
      ***/

      //      cycleTime = 4 * (10-level) * (10-level) + 10;      // ~ 400-40
      //      cycleTime = ((10-level) * (10-level) * (10-level))/2 + 20;      // ~ 800-50 is OK
      cycleTime = 5 * (10 - level) * (10 - level) + 20;  // ~ 500-40 ?

      if (DispScanWait(cycleTime)) {
        if (btn1Pressed || btn4Pressed) {
          if (ver == 1 || ver == 3) targCell = scanCell;
          else        targCell = (scanCell + (nleds / 2)) % nleds + 1;
          if (cell[targCell] == 1) {
            cell[targCell] = 0; // got it
            beepTone(1200,8); beepTone(1600,16); // p-ing off the target
          } else {
            cell[targCell] = 1;
            buZZ(10);       // missed
          }
        }
        if (ESC) {
          refreshWait(300);
          return;
        }
        if (btn3Pressed) {    // step up the level
          level = (level >= 10) ? 1 : level + 1;
          announceLevel(level);
        }
      }

    }
  }
}

// =====================================================================
void JumpMan() { // Jump Man - as objects approach jump them or duck under them
  byte csum;
  bool jump, duck;
  bool jumping;
  bool jBtn, dBtn;
  bool triedLast, lastDBtn;
  int cycleTime;
  int objCnt;
  bool playing = true;
  byte ver = 1;

  if (nleds < 8) {
    ver = 1;
  } else {
    ver = getOption(3);  // version 1: will use button 1 or 4 to jump
    // version 2: add over head obsticles, to duck under (if you figure out how)
    // version 3: for Jump: Btn1 / Duck: btn2 or btn4
  }
  if (ver == 0) return;

  for (int i = 1; i <= nleds; i++) {
    cell[i] = 0;
  }

  level = 1;
  objCnt = 0;
  jump = duck = false;
  playing = true;
  while (playing) {

    if (cell[1] == 1) {
      if (jump) { // jump btn must be newly press (or in a multi-object jump
        // GOOD
        jumping = true;
        if (cell[2] == 0) objCnt++; // multi cell objects are credited only once
      } else { // BAD
        objCnt--;
        beep(25);
      }
    } else {
      jumping = false;
    }

    if (cell[1] == 2) {
      if (duck) { //
        // GOOD
        if (cell[2] == 0) objCnt++;
      } else { // BAD
        objCnt--;
        beep(25);
      }
    }
    if (objCnt >= 4) {
      objCnt = 0;
      level++;
      // acknowledge the Advancement
      beep(150); refreshWait(50); beep(350);
      // clear cell 1-(nleds/2)
      cell[1] = cell[2] = cell[3] = cell[4] = cell[nleds / 2 - 1] = cell[nleds / 2] = 0;
    }
    if (objCnt < 0) objCnt = 0;

    csum = 0;
    // shift obsticals left
    for (int i = 1; i < nleds; i++) {
      cell[i] = cell[i + 1];
      if (i > (nleds / 2)) csum += cell[i];
    }

    if (random(0, 6) >= 5) cell[nleds] = 1; // 1/6 of the time generate an obstical
    //    if (random(0,5)>=4) cell[nleds] = 1;  // 1/5 of the time generate an obstical
    else cell[nleds] = 0;
    if (ver > 1 && random(0, 6) >= 5 && cell[nleds] == 1 && cell[nleds - 2] != 2 && cell[nleds - 1] != 2)  cell[nleds] = 2;
    if (csum == 0 && cell[nleds] == 0) cell[nleds] = random(0, ver + 1); // if low on objs try to add something in

    //================ UPDATE Display
    spriteOne = 0;
    flashLed = 0;
    for (int i = nleds; i >= 1; i--) {
      dim[i] = (cell[i] != 0);
      if (cell[i] == 2) {
        flashLed = i;
      }
    }
    dim[1] = true;
    if (cell[1]) spriteOne = 1;

    // cycleTime = ((10-level) * (10-level) * (10-level))/2 + 20;      // ~ 800-50 is OK
    // cycleTime = (5 * (12 - level) * (12 - level)) + 30;  // ~
    if (level<7) cycleTime = (5 * (12) * (11 - level));
    else         cycleTime = (5 * (14-level) * (14-level));
    
    DispScanWait(cycleTime);  // refresh display while catching btn press
    // what is the player trying to do ?
    triedLast = jBtn;
    if (ver < 3) {
      jBtn = btn1 || btn4;
      duck = triedLast && !BtnPressed;
    } else {
      jBtn = btn1;
      lastDBtn = dBtn;
      dBtn = btn2 || btn4;
      duck = dBtn && !lastDBtn;
    }
    jump =  jBtn && (!triedLast || jumping);

    if (btn3Pressed) {    // step up the level
      level = (level >= 9) ? 1 : level + 1;
      announceLevel(level);
    }
    // provide for changing ver while playing
    if (btn3) {
      ver = ver % 3 + 1;
      level = 1; // would user rather not reset the level(speed) ?Hum
      beeps(50, ver); refreshWait(500);
    }

    // if 'ESC' ... Exit, but first report on level reached
    if (ESC) {
      showLevel(level);
      refreshWait(750);
      return;
    }
    if (level > 12) level = 1; //   >=12 is 'plaid' speed
  }
}

void Skee_ball() { // Roll a set of balls into target "hoops" for maximum score
  // three target "hoops", worth (1) (3) (2)  pts. The closest is the 1pt hoop
  // You are scored after each 5 balls (15 is the maximum score)
  // hold Btn1 to swing, and let go to throw the ball
  int player = 0;
  int i, cnt;
  byte spot, cell, balls, score;
  byte h1, h2, h3;  // hoops valued 1-3
  int travel, rate, thrust;
  bool rolling, newPressed, released, btnDown, wasDown;

  loopCnt = 0;
  level = 1; // starts at 1, increase each time you get 10+ pts
  ver = 1;  //if (ver==0) return;
  glowEnable = true;

  clearDisp(); refreshWait(500);
  while (1) {
    // setup game-level
    wait_BtnRelease();   // wait for no button pressed
    spot = 1;
    cell = balls = score = 0;
    clearDisp();
    glow[nleds - 2] = true;
    dim[nleds - 3] = lit[nleds - 1] = true;
    h1 = nleds - 3; h3 = nleds - 2; h2 = nleds - 1;
    playing = true;

    while (playing) {
      spot = 1;       // where we are throwing from
      dimLED = 1;
      travel = 0;
      thrust = 0;
      rolling = false;
      turn = 1;   // # of rolled balls
      while (turn) {
        bool pressed = btnPressed();
        wasDown = btnDown; // keep B2&3 from wasting a 'release'
        btnDown = (btn1 || btn4);
        if (btnDown && !rolling) {
          thrust++;
          cell = 1;
          spriteOne = 1;
        }

        released = wasDown && (!btnDown) && BtnChanged;
        if (released) {   // the ball takes flight
          rolling = true;
          dimLED = 0;
          thrust = (3 * thrust) / (level + 2); // at higher levels targets are effectivly further, require more thrust
          //Serial.print("thrust: "); Serial.println(thrust);
          rate = (thrust + 750) / 15; //for thrust=7500 gives 100 which results in 10 spots in one second
          rate = rate / 4; // patch so we can update more often for smoother fast motion balls
        }

        if (rolling && (loopCnt % 50) == 0) { // was %200 before patch above
          if (thrust <= 30) {         // coming to a stop
            if ((nleds - 3) <= cell && cell <= (nleds - 1)) { // goes in one of the hoops
              score += (cell == h3) ? 3 : (cell == h2) ? 2 : 1;
              beep(40, 15);   // Tah
              if (cell == h2) beep(30);
              if (cell == h3)  {
                refreshWait(130);
                beep(200);   // Dah
              }
            } else {      // missed
              buZZ(50);
            }
            turn = 0;
            balls++;
            refreshWait(1000);
            cell = 0;
            spriteOne = 0;

            if (balls >= 5) {  // 5 ball set complete, give score
              showLevel(score);   // show the Score
              refreshWait(2000);
              if (score > 10) level++;
              playing = false;
              break;  // from this game-set
            }
            continue;
          }

          travel = travel + rate;
          thrust -= rate;
          cell = spot + travel / 100;
          if (cell >= nleds + 1) cell = (cell % nleds) + 1;
          spriteOne = cell;
        }

        //================ UPDATE Display
        loopCnt++;
        refreshWait(1);
        // if 'esc' button held down ... Exit
        if (ESC || (btn4 && btn1)) {
          return;
        }
      }
    }
  }
}

void Sea_Hunt() { // find enemy with Sonar and destroy with Depth Charges
  // reworked for use with 8 instead of 12 LEDs
  int targCell;
  int subLoc, subDir;
  int player = 1; // first player is #1 second is #0
  int depChrg = 0;
  int depChgCnt, depChgLimit;
  bool sonarPing = false;
  byte ping;
  bool advance;
  byte n;

  ver = getOption(4); // version 1: Each volley is both a depth-charge and a sonar-ping
  // version 2: Head to head, taking turns, to see who sinks the sub (player1 uses Btn1 the other Btn4)
  // version 3: depth-charges and sonar-pings are launch separately (via Btn1 & Btn4)   3 depth-charges only, if enabled
  // version 4: the sub speed can be 1-3 cells per volley,                              6 depth-charges only
  // a limit to the number of depth charges for ver. 3 & 4 can be activated
  if (ver == 0) return;
  while (1) {
    // setup game-level
    clearDisp();
    refreshWait(750);

    // locate sub & its direction
    subLoc = random(1, nleds + 1);
    subDir = (random(0, 2) == 0) ? -1 : 1;
    depChgLimit = (ver < 4) ? 3 : 6;
    if (ver == 4) subDir = subDir * random(1, 4); // speed can be +/- 1,2,3
    //    Serial.print("subLoc: "); Serial.print(subLoc); Serial.print("    subDir: ");  Serial.println(subDir);

    wait_BtnRelease();   // wait for no button pressed
    targCell = nleds / 2;
    depChgCnt = 0;
    playing = true;

    while (playing) {
      advance = false;
      if (btnPressed()) {

        // if 'esc' button held down  ... Exit
        if (ESC) {
          delay(300);
          return;
        }

        if (btn2Pressed) targCell--;
        if (btn3Pressed) targCell++;
        if (targCell <= 0) targCell = nleds;
        if (targCell >= nleds + 1) targCell = 1;
        if (ver == 1) {
          depChrg   = btn1Pressed || btn4Pressed;
          sonarPing = depChrg;
        } else if (ver == 2) {
          depChrg   = (player == 1 && btn1Pressed) || (player == 0 && btn4Pressed);
          sonarPing = depChrg;
        } else if (ver == 3 || ver == 4) {
          depChrg   = btn1Pressed;
          sonarPing = btn4Pressed;
          //                        activate the next line if you want to limit the # of depth charges
          if (depChrg) depChgCnt++;    // only tracked for ver 3&4
        }

        if (ver == 2 && ((btn4Pressed && player == 1) || (btn1Pressed && player == 0))) {
          beep(25);
        } else {
          if (depChrg) {
            advance = true;
            // drop depth-charge
            //deChrg = 20;
            spriteTwo = litLED = targCell;
            refreshWait(250);
            boop(30);               // sound of charge
            refreshWait(250);
            spriteTwo = litLED = 0;
            if (subLoc == targCell) {
              refreshWait(200);
              spriteOne = targCell;
              refreshWait(150);
              boOop(250);            // ka ...
              spriteOne = 0;
              dim[targCell - 1] = dim[targCell + 1] = true;
              Beep(500);             // ... Bang!
              dim[targCell - 1] = dim[targCell + 1] = false;
              refreshWait(750);
              if (ver == 2) winner(player);
              refreshWait(750);
              playing = false;
            } else {  // missed
              if (depChgCnt >= depChgLimit) {
                lost();   // player has lost, give the 'Raspberry
                playing = false;
              }
            }
          }
          
          if (sonarPing && playing) {      // take sonar reading
            advance = true;
            ping = dimIt = flashLed = 0;
            n = (nleds < 8) ? 1 : (nleds < 12) ? 2 : 3;
            for (int i = 0; i <= n; i++) {
              if ((targCell - i) > 0) {
                dim[targCell - i] = true;
                if ((targCell - i) == subLoc) ping = targCell;
              }
              if ((targCell + i) < nleds + 1) {
                dim[targCell + i] = true;
                if ((targCell + i) == subLoc) ping = targCell;
              }
              refreshWait(150);
              if (ping == targCell) beep(25);
              refreshWait(150);
              if ((targCell - i) > 0) dim[targCell - i] = false;
              if ((targCell + i) < nleds + 1) dim[targCell + i] = false;
              ping = litLED = flashLed = 0;
            }
          }
        }
        if (advance) {
          player = 1 - player;
          // the sub moves after being poked at
          subLoc = subLoc + subDir;
          if (subLoc < 1)  subLoc += nleds;
          if (subLoc > nleds) subLoc -= nleds;
          //Serial.print("subLoc: "); Serial.println(subLoc);
        }
      }

      if (ESC || (btn4 && btn1)) {
        delay(300);
        return;
      }
      //================ UPDATE Display
      spriteOne = spriteTwo = dimIt = litLED = flashLed = 0;
      //      spriteTwo = subLoc; // do this to debug
      flashLed = targCell;
      //dimLED = dimIt = targCell;   // too soften the flash
      litLED = dimIt = targCell;   // too soften the flash (removed dim+dimIt due to flicker)
      refreshWait(1);

    }
  }
}

// =====================================================================
void BiFunc() {  //  Binary Operator Functions - deduction challenge
  int i, csum;
  int testCell, targCell;
  byte opr1, opr2, results;

  loopCnt = 0;
  ver = getOption(4);  // Funcunctions 1: AND   2: OR   3: XOR   4: ADD
  if (ver == 0) return;
  clearDisp();

  refreshWait(500);
  while (1) {
    // setup game-level
    ledsAll(0); refreshDisp();  // clear all 'lit[]' leds

    wait_BtnRelease();   // wait for no button pressed

    targCell = 4;
    playing = true;
    opr1 = opr2 = 0;
    while (playing) {
      if (btnPressed()) {
        if (btn2Pressed) targCell--;
        if (btn3Pressed) targCell++;
        if (targCell <= 0) targCell = 4;
        if (targCell >= 5) targCell = 1;
        if (btn2Pressed || btn3Pressed) loopCnt=0; // causes flash to indicate new pos.
        if (btn1Pressed) {
          opr1 = opr1 ^ (1 << (4 - targCell));
        }
        if (btn4Pressed) {
          if (opr1 == 0 || opr1 == 0xF) { // initial opr1, used by the player must != 0, 15
            beep(20);
            continue;
          }
          if (opr2 == 0) { // if we have no opr2 yet, pick one that != opr1
            while (opr2 == 0 || opr2 == opr1) opr2 = 1 + random(14);
          }
          switch (ver) {
            case 1: results = opr1 & opr2;  break;
            case 2: results = opr1 | opr2;  break;
            case 3: results = opr1 ^ opr2;  break;
            case 4: results = opr1 + opr2;  break;
          }
          //Serial.print("opr1: "); Serial.print(opr1); Serial.print("  opr2: "); Serial.print(opr2);
          //Serial.print("   results: "); Serial.println(results);


          if (opr1 == opr2) { // you got it
            winner(1);
            refreshWait(500);
            playing = false;
          } else {
            buZZ(150);  //boo
            targCell = 0;
            // show the Bi Operation results
            opr1 = (results & 0x0F);
          }
        }
      }

      //================ UPDATE Display
      loopCnt++;
      dimLED = litLED = 0;

      for (i = 0; i < 4; i++) {
        colorLeds[4 - i] = (opr1 >> i) & 1;
      }
      if ((loopCnt % 811) < 55) { // creats cursor effect
        colorLeds[targCell] = !colorLeds[targCell];
      }

      dspColors();  // takes up 1 msec
      refreshWait(1); // keep from scanning btns too fast

      if (ESC || (btn1 && btn4)) {
        buZZ(30);
        refreshWait(300);
        return;
      }
    }
  }
}

void Flip_d() {   // One dimensional, "Reversi" game. Reminiscent of 'Go'
  int targs, testCell, targCell;
  int csum;
  int lastCell, aiMove, hisFirstLast, hisLastFirst, myLastFirst, mySplit;
  byte r, x, myGrps, hisGrps, setSize;
  byte color, moves;

  loopCnt = 0;
  ver = getOption(2);  // version 1: player 1 vs Computer challenge
  // version 2: two person
  if (ver == 0) return;
  clearDisp(); refreshWait(1000);
  player = 0;
  while (1) {
    // setup game-level
    //clearDisp(); refreshDisp();
    clearDisp();

    for (int i = 1; i <= nleds / 2; i++) { // select half the leds, randomly
      r = random(nleds) + 1;
      while (dim[r]) {
        r = random(nleds) + 1;
      }
      dim[r] = true;
    }
    wait_BtnRelease();   // wait for no button pressed

    aiMove = 0;
    targCell = nleds / 2;
    playing = true;
    moves = 0;
    if (ver != 2) player = 0;
    while (playing) {
      idleCnt = 0; // defeat idle timeout while playing this comtemplated game
      if (btnPressed() || aiMove) {
        if (ESC) return;
        if (btn2Pressed) targCell--;
        if (btn3Pressed) targCell++;
        if (targCell <= 0) targCell = nleds;
        if (targCell >= (nleds + 1)) targCell = 1;
        if (ver == 1 && moves == 0 && btn4Pressed && player == 0) {
          player = 1;         // let the AI go first
          btn4 = false; //stop further btn event processing
        }
        if ((btn1Pressed && player == 1) || (btn4Pressed && player == 0)) {
          beep(30);
        }
        else if (btn1Pressed || btn4Pressed || aiMove)
        { // -------------------  make their move
          if (dim[targCell] != player) { // convert the other guys set to your side
            testCell = targCell;
            while (dim[testCell] != player && testCell <= nleds) {
              dim[testCell] = player;
              testCell++;
            }
            testCell = targCell - 1;
            while (dim[testCell] != player && testCell > 0) {
              dim[testCell] = player;
              testCell--;
            }
          } else {  // it is one of yours to be given up
            dim[targCell] = 1 - player;
          }
          csum = 0;
          for (int i = 1; i <= nleds; i++) {
            if (dim[i]) csum ++;
          }
          if (csum == 0 || csum == nleds) { // game over
            color = (csum == 0) ? 1 : 4;
            // acknowledge the "Win-ner!"
            beep(200);
            refreshWait(75);
            setColor(color); beep(500);
            setColor(0);
            refreshWait(200);
            playing = false;
          }
          moves++;
          player = 1 - player;
          dimFlash = litLED = 0;
          boop(20);
          refreshWait(750); // for viewing new state
        }

        //  ------------------------- support playing against the Computer
        lastCell = -1;
        hisGrps = myGrps = 0;
        aiMove = hisFirstLast = hisLastFirst = myLastFirst = mySplit = 0;
        if (playing && ver != 2 && player == 1) { // AI looks for a move
          for (int i = 1; i <= nleds; i++) { // tally the board position
            x = dim[i];
            if (x != lastCell) {
              if (x) myGrps++; else hisGrps++;
              if (x == 1 && hisFirstLast == 0) hisFirstLast = i - 1;
              if (x == 0) hisLastFirst = i;
              if (x == 1) myLastFirst = i;
              setSize = 1;
            } else {
              setSize++;
              if (x && setSize == 3) mySplit = i - 1;
            }
            lastCell = x;
          }
          // -------------- decide on move
          if (hisGrps == 1) aiMove = hisLastFirst;
          else if (myGrps == 1) aiMove = mySplit;
          else if (hisGrps == 2) aiMove = (dim[1] == 0) ? hisFirstLast : aiMove = hisLastFirst;
          else if (myGrps == 2) aiMove = (mySplit) ? mySplit : myLastFirst;
          else if (hisGrps == 3) aiMove = (dim[1] == 0) ? hisFirstLast : aiMove = hisLastFirst;
          else if (myGrps < hisGrps) aiMove = (mySplit) ? mySplit : myLastFirst;
          else {
            if (dim[1] == 0) aiMove = hisFirstLast;
            if (dim[nleds] == 0) aiMove = hisLastFirst;
          }
          if (aiMove == 0)  aiMove = random(nleds) + 1;
        }
      }


      //================ UPDATE Display
      //flashLed = (player==1)? targCell:0 ;
      loopCnt++;
      dimFlash = litLED = 0;
      if (player == 0) {
        if ((msCnt/100)%2==1) dimFlash = targCell; // 1/2 the effect of dimFlash
      }
      if  (player != 0 && loopCnt < 350) {
        litLED = targCell;
      }
      if (loopCnt > 600) loopCnt = 0;

      refreshWait(1);
      if (aiMove) {
        dimFlash = litLED = 0;
        refreshWait(1000);
        targCell = aiMove;
        flashLed = aiMove;
        refreshWait(1500);
        flashLed = 0;
      }

      // if 'esc' button held down or Fnc-1 ... Exit
      if (ESC) {
        refreshWait(300);
        return;
      }
    }
  }
}

// =====================================================================
void Spray() {  // Out spray paint the other guy
  byte  litCnt, fullP;
  short int paint, aim, msDelay;
  bool sprayIt;
  byte newGame, leadPlayer;
  int i;
  bool pressed;
  byte levelMax;
  byte player;

  clearDisp();
  levelMax = 10;
  loopCnt = 0;
  player = leadPlayer = 1;
  dir = 1;
  aim = -3;
  fullP = nleds / 2; // std units of paint

  ledsAll(0); refreshWait(500);
  paint = fullP + 1; // initial an extra unit of paint
  while (1) {
    // setup game
    level = 1;
    ledsAll(0); refreshDisp();
    loopCnt = 0;
    sprayIt = false;
    newGame = false;
    if (aim != -3) { // if not initial round
      paint = -1;
      newGame = true;
      if (dir == 1) {
        aim = nleds + 3;
        leadPlayer = 2;
      } else {
        aim = -2;
        leadPlayer = 1;
      }
    }

    wait_BtnRelease();   // wait for no button pressed
    refreshDisp();
    playing = true;
    while (playing) {

      //================ Advance Game Status
      loopCnt++;
      msDelay = (level <= levelMax / 2) ? 125 - 12 * level : 100 - 8 * level;
      if ((loopCnt % msDelay) == 0) {

        // ------ Is the game over ?
        litCnt = 0;
        for (i = 1; i < nleds + 1; i++) {
          litCnt += dim[i];
        }
        if ((paint != -1 && paint < fullP) && (litLED == aim) && ((player == 1 && litCnt == nleds) || (player == 2 && litCnt == 0))) { // ? all one color
          // winner
          winner(player);
          refreshWait(1000);
          playing = false;
          newGame = true;
        }

        aim = aim + dir;
        if ((aim <= -2) || (aim >= nleds + 3)) { // at the end, turn around
          dir = (aim <= -2) ? 1 : -1;
          if (paint < fullP) { // done his thing (paints been used), switch players
            paint = (paint < 0) ? fullP + 1 : fullP;
            player = 3 - player;
            if (player == leadPlayer) {
              level++;
              if (level > levelMax) level = 1;
            }
            refreshWait(330);
            spriteOne = (player == 1) ? 1 : nleds;
            beep(15);
            refreshWait(150);
            spriteOne = 0;
            refreshWait(330);
            if (newGame) {
              newGame = false;
              for (int n = 1; n <= nleds; n++) dim[n] = (player != 1);
            }
          }
        }
      }

      //================ UPDATE Display
      if (aim > 0 && aim < nleds + 1)  litLED = aim;
      else litLED = 0;
      refreshWait(1);

      //================ Process Keys
      if (btnPressed() || (loopCnt % msDelay) == 0) {
        // qualify button press,  p1 btn1, p2 btn4 or btn0
        pressed = ((player == 1) && btn1) || ((player == 2) && btn4);

        sprayIt = (paint && pressed);

        if (sprayIt) {
          paint--;
          if (aim > 0 && aim < nleds + 1) {
            // see if spot not already painted by him
            if (dim[aim] != (player == 1)) {
              dim[aim] = (player == 1);
            } else {
              if (paint == (fullP - 1)) beep(20); // boo, they missed right off
              paint = 0;
            }
          }
        }
      }

      if (btn3Pressed) {    // step up the level
        level = (level >= levelMax) ? 1 : level + 1;
        announceLevel(level);
      }
      if (ESC || (btn1 && btn4)) {
        refreshWait(300);
        return;
      }
    }
  }
}

//=============================================================
// mostly from Menu16Games Set #2
// ************************  Group 2  *******************************

// =====================================================================
void pingPong() { // Ping-Pong - Two player Game
 #define STARTPERIOD 330;
  int litPeriod, period;
  int cnt = 0, maxCnt;
  int rally = 0;
  int dir = 1;
  bool slam;
  bool isLit = false;
  bool lastSwung, swing = false;
  bool lastPosHit, hit = false;
  int ver;

  ver = 2;    // version 2: for 2 players P1 must use Btn1, P2: btn4
  //  ver = getOption(2); // version 1: use either Btn1 or 4 good for one player
  litPeriod = STARTPERIOD;
  period = litPeriod;
  maxCnt = nleds - 1;
  t0 = millis();

  while (1) {
    lastSwung = swing;
    scanBtns();
    if (ESC) return;
    swing =  btn1Pressed || btn4Pressed;
    if (ver == 2) {
      if (dir == 1) swing = btn4;
      else        swing = btn1;
    }
    hit = hit || swing;
    if (btn3Pressed) {    // step up the speed/level
      rally = rally + 1;
      litPeriod = STARTPERIOD;
      for (int i = 0; i < rally; i++) litPeriod = 9 * litPeriod / 10;
      cnt = (dir==1) ? 0 : nleds - 1; // restart the rally
      isLit = false;
    }

    if ((millis() - t0) >= period) {  // is this time period up ?
      if (isLit) {                       // about to go from LED on to OFF
        //if (cnt==5) {Serial.print("Period was: ");  Serial.println(period);} 
        period = litPeriod / 3;
          if (cnt >= maxCnt || cnt <= 0) { // ------------ end of the line
            litPeriod = STARTPERIOD;
            for (int i = 0; i < rally; i++) {
              litPeriod = 9 * litPeriod / 10;   // * .9
            }

            //        !!!!!!!!!!!!!!!! ------  make judgement
            if (hit ^ lastPosHit) {
              // -------------------  they hit it
              litLED = cnt + 1;
              //refreshWait(40);
              slam = lastPosHit;
              if (slam) litPeriod = 6*litPeriod/10;  // slam-hit it
              bop(50 + slam*100);  // paddle hitting ball
              litLED = 0;
              hit = false;
              rally++;
              dir = -dir;
              cnt = (dir==1)? 0 : nleds - 1;
            } else {  // ------------  failed to hit it
              litLED = 0;
              clearDisp();
              // -----  acknowledge the "Win-ner!"
              if (rally>0) {
                if (dir > 0) {
                  brt[1] = 1;
                  beep(150,100);
                  brt[2] = 1;
                  beep(500);
                }
                else          {
                  brt[nleds - 1] = 1;
                  beep(150,100);
                  brt[nleds] = 1;
                  beep(500);
                }
              }

              refreshWait(1000);      // that round is over ...
              if (rally >= 2) {
                showLevel(rally / 2); // give em one star for each set of back & forth
                refreshWait(2000);
                brightOne = 0;
                refreshWait(200);
              }

              // re-init for a new round
              litPeriod = STARTPERIOD;
              period = litPeriod;
              rally = 0;
              cnt = (dir==1)? 0 : nleds - 1;
              clearDisp();
            }
          } // end time to judge
      } else {                     // about to go from LED off to ON
        period = litPeriod;
        lastPosHit = hit;
        hit = false;
        cnt = cnt + dir;  // goto NEXT cell
      }
      t0 = millis();
      isLit = not isLit;
    }

    //-----------  UPDATE Display
    for (int i = 1; i <= nleds; i++)  dim[i] = 0;
    dim[cnt + 1] = 1;
    refreshWait(1);
    if (ESC) return;
  }
}

void QuickDraw() {  //  Ready ... Set ... Draw !
  int ret, who;
  bool playing;

  clearDisp(); refreshWait(750);
  setColor(5); refreshWait(500);    // indicates we have selected the game

  while (1) {
    // setup game-level
    setColor(0); refreshWait(1000);

    playing = true;
    while (playing) {

      while (!btnPressed()) {   // pause till player gives go ahead, with two Leds dim flashing
        dimFlash = nleds - (msCnt % 2);
        refreshWait(1);
      }
      dimFlash = 0;

      while (Btn) {    // wait for NO button pressed
        scanBtns();
        refreshWait(1);
        if (ESC && !btn1) return;
      }

      refreshWait(700);
      ret = readySetGo(2000); // There is a Random time between yellow and 'Draw!' green
      t0 = millis();  // start timing on  Button pushed
      if (ret == 0) {
        while (!btnPressed()) {}  // Wait for the First press
        t2 = millis() - t0;
        Serial.print("Reaction time = ");  Serial.print(t2); Serial.println("ms");
        if (btn2) {
          refreshWait(500);
          if (t2 > 500) {
            Beep(50);
            break;
          }
          showLevel(t2 / 20); // show 1 LED for each 20ms
          refreshWait(2000);
          clearDisp();
        }
        // if 'esc' button held down or Fnc-1 ... Exit
        if ((ESC && !btn1) || btn3) {
          refreshWait(300);
          btn3 = 0;
          return;
        }

        if (btn1Pressed && btn4Pressed) { // TIE
          winner(-1);
        }
        if (btn1Pressed || btn4Pressed) { // we have a winner
          winner(btn1Pressed);
        }

      } else {        // there has been a false start, shame on him
        who = (btn1) ? 1 : (btn4 ? 4 : 0);
        setColor(who); beep(200);
        setColor(0);
        scanBtnWait(1000);   // then try again
      }
    }
  }
}

// =====================================================================
void Tug_a_War() { // Tug (with faster clicks) more than the other to pull in a win 
  int ret;
  int player1Cnt, player2Cnt;
  int winLen, bowLoc, loc;  // center Bow location

  //byte ver = getOption(2);  // version 1: could be Fixed time between Set ... Go!
  // version 2: could be Random time between yellow and green
  clearDisp(); refreshWait(750);
  setColor(5); refreshWait(500);    // indicates we have selected the game

  while (1) {
    // setup game-level
    winLen = (2 * (nleds / 2 + 2));
    clearDisp(); refreshWait(1000);

    while (!btnPressed()) {   // pause till player gives go ahead, with two Leds dim flashing
      dimFlash = nleds - (msCnt % 2);
      refreshWait(1);
    }
    dimFlash = 0;

    while (btnDown()) {
      refreshDisp();  // wait for NO button pressed
      delay(1);
      if (ESC && !btn1) return;
    }
    refreshWait(700);
    playing = true;
    while (readySetGo(400)) ;  // false start returns 1 else 0
    refreshWait(250);  // allows the 'Go' green to be seen
    // and allow players to get going, so we are measuring their rate of button pressing not their reaction time
    clearDisp();
    player1Cnt = player2Cnt = 0;
    while (playing) {
      scanBtns();
      if (ESC && !btn1) {
        delay(300);
        return;
      }
      if (btn1Pressed) {
        player1Cnt++;
        if ((player1Cnt - player2Cnt) >= winLen) {
          winner(1);
          break;
        }
      }
      if (btn4Pressed) {
        player2Cnt++;
        if ((player2Cnt - player1Cnt) >= winLen) {
          winner(2);
          break;
        }
      }

      //================ UPDATE Display
      if (btnPressed) clearDisp(); // clear all leds light settings
      bowLoc = (nleds / 2) + (player2Cnt - player1Cnt) / 2; // calculate the Location of the Bow tied on the line
      loc = bowLoc - 1; if (0 < loc && loc <= nleds) dim[loc] = true;
      loc = bowLoc;   if (0 < loc && loc <= nleds) spriteOne = loc;
      loc = bowLoc + 1; if (0 < loc && loc <= nleds) spriteTwo = loc;
      loc = bowLoc + 2; if (0 < loc && loc <= nleds) dim[loc] = true;
      refreshWait(1);  // BTW: the use of (2) creates a race condition where a fraction of the time
      // with RP2040 dim[] LED updates are ineffective ??!! ...resolved with a patch
    }
    if (ESC && !btn1) {
      delay(300);
      return;
    }
  }
}

// =====================================================================
void Chicken() {  // Don't bail to win. But if you collide you lose.
  int ret;
  int p1Loc, p2Loc, p1jumped, p2jumped;
  int winningPlayer;
 #define MOVE_FREQ 250
  int move_freq, moveCnt;
  bool ledFlip;

  byte ver = 1;
  //byte ver = getOption(2);  // version 1:
  // version 2:  TBD
  move_freq = MOVE_FREQ;

  // indicates we have selected this game
  for (int i = 0; i < nleds; i++) {
    dim[i + 1] = true; dim[nleds - i] = true;
    DispScanWait(move_freq / 2);
    dim[i + 1] = false; dim[nleds - i] = false;
  }

  while (1) {
    scanBtns();
    while (btn4) {
      scanBtns();
      refreshWait(1);
      if (ESC && !btn1) return;  // only exit opertunity
    }

    while (!(btn1 && btn4)) {   // till both players press down, dim flashing prompt LEDs
      scanBtns();
      ledFlip = ((msCnt / (2 * move_freq)) % 2) == 0;
      dim[1] = ledFlip;
      dim[nleds] = !ledFlip;
      spriteOne = btn1;
      spriteTwo = (btn4) ? nleds : 0;
      refreshWait(1);
      if (btn3Pressed) {                // at this time press Btn3 to increase &
        move_freq = 4 * move_freq / 5;              // recalculate speed
        bop(30);
      }
    }

    // setup game-level
    loopCnt = 0;
    p1Loc = p2Loc = 0;
    p1jumped = p2jumped = 0;
    winningPlayer = 0;

    bop(100);
    refreshWait(random(150, 350)); // ready state to be watched
    clearDisp();

    moveCnt = 0;
    playing = true;
    while (playing) {
      loopCnt++;
      moveCnt = (moveCnt + 1) % move_freq;

      if (p1Loc > 0) {
        if (!btn1 && p1jumped == 0) { // if released log it
          p1jumped = loopCnt;
        }
        if (moveCnt == 0) p1Loc++;  // move it ahead
        if (p1Loc > nleds) winningPlayer = 1; // he reach the other end
      } else {
        // if btnpressed start blip on its way
        if (btn1) p1Loc = 1;
      }

      if (p2Loc > 0) {
        if (!btn4 && p2jumped == 0) { // if released log it
          p2jumped = loopCnt;
        }
        if (moveCnt == 0) p2Loc--;
        if (p2Loc == 0) winningPlayer = 2;
      } else {
        if (btn4) p2Loc = nleds; // if btnpressed start blip on its way
      }

      // display update
      clearDisp();   // clear out old states
      spriteOne = p1Loc;     //  note an 'litLED' and a 'dimLed'+'bonus' are the same brightness
      spriteTwo = p2Loc;
      refreshDisp();

      // if blips collide or both released
      if ((p1Loc >= p2Loc && p2Loc != 0) || (p1jumped && p2jumped)) { // if blips collide or both released
        //Serial.print(p1jumped); Serial.print("  "); Serial.println(p2jumped);
        if (p1jumped == p2jumped) winningPlayer = -1; // its a tie
        else winningPlayer = (p1jumped > p2jumped) ? 1 : 2; // its over. Who won ?
      }
      if (winningPlayer != 0) winner(winningPlayer); // game will restart

      refreshWait(1);
      scanBtns();

      //  not good here:     if (ESC) return;
      if (btn1) escCnt = msCnt; // reset cnt for ESC, so B4 must be held alone
    }
  }

}

// =====================================================================
void Hot_Hands() { //  Recreation of the kid's hand slapping contest
  // Two players press down their buttons together, one side releases then tries to press his button twice
  //  before the other side can release his button. 
  // The player's turn is indicated at the start of a round by a lighting of the leds on his side.
  int ret;
  int ledSet;
  int p1Trys, p2Trys;
  int player = 1;
  int who;
  bool escGo;

  clearDisp(); refreshWait(750);
  ledsAll(15); refreshWait(500);    // indicates we have selected the game

  byte ver = 1;
  //  byte ver = getOption(2);  // version 1: They take turns trying to slap the other
  //TBD: if desired:
  // version 2: Free for all. Either can slap at will
  
  while (1) {
    // indicate who's turn it is to try and slap it to the other
    who = (player == 1) ? 1 : 8;
    ledsAll(who); DispScanWait(500);
    // setup game-level
    ledsAll(0); DispScanWait(1000);

    while (btnPressed()) {
      refreshDisp();  // wait for NO button pressed
      delay(1);
    }

    playing = true;
    while (playing) {

      while (!(btn1 && btn4)) {
        scanBtns();
        if (ESC && !btn1) {  // double how long before b4->Exit
          if (escGo) return;
          escGo = true;
          escCnt = msCnt;
        }
        ledSet = (btn1) ? 1 : 0;
        ledSet = (btn4) ? 8 : ledSet;
        ledsAll(ledSet);
        refreshDisp();
        delay(1);
      }
      escGo = false;
      ledsAll(9); // both Red & Blue leds on

      while (btn1 && btn4) { // Wait for the First release
        refreshWait(1);
        scanBtns();
      }
      if ((player == 1 && !btn4) || (player == 0 && !btn1 )) { // wrong player released first
        beep(25);
      } else {
        p1Trys = 0; p2Trys = 0;
        while (playing) {
          if (! (btn1 || btn4)) {  // if both have released, the attempt has Failed
            ledsAll(0);  refreshDisp();
            buZZ(100);
            playing = false;
            refreshWait(500);
          }

          if (btn1Pressed) p1Trys++;
          if (btn4Pressed) p2Trys++;

          if (p1Trys > 1) {
            // if the other player's button is still down they Win
            if (btn4) winner(1);  // resets 'playing'
            else      winner(2);
          }

          if (p2Trys > 1) {
            if (btn1) winner(2);
            else      winner(1);
          }

          // update display
          ledSet = (btn1) ? 1 : 0;
          ledSet = (btn4) ? ledSet + 8 : ledSet;
          ledsAll(ledSet);
          refreshDisp();
          delay(1);
          scanBtns();
          // to 'esc' double how long before b4->Exit  see above
        }
      }
    }
    player = 1 - player;
    ledsAll(0); refreshWait(1000);
  }
}

// =====================================================================
void PIG() {    // 'PIG' basketball,  3 misses and you're done
  int player = 0;
  int csum;
  byte shot, spot, dribbleCnt, score, pigCnt;
  int travel, rate, thrust;
  bool inAir, newPressed, released, btnDown;
  unsigned int lastLCnt;
  byte targCell;

  loopCnt = 0;
  ver = getOption(2);  // version 1: rotate shot 1,2,3   count how many made before U have 'PIG'
  // version 2: 2 player, player picks shot, first to 'PIG' loses
  if (ver == 0) return;

  clearDisp(); refreshWait(500);
  targCell = nleds - 1;

  while (1) {
    // setup game-level
    wait_BtnRelease();   // wait for no button pressed
    shot = 0;
    spot = 1;
    score = pigCnt = 0;
    playing = true;
    while (playing) {

      clearDisp(); refreshDisp();
      shot = shot % 3 + 1; // 1= the first spot is the 3pt. line, 2= free throw, 3= lay-up
      // w/12LEDS:      if (ver==1) spot = shot+shot-1;
      if (ver == 1) spot = shot;
      turn = shot;
      dim[targCell] = true;
      dimFlash = spot;

      dribbleCnt = 0;
      lastLCnt = 0;
      travel = 0;
      thrust = 0;
      inAir = false;
      turn = shot;
      while (turn) {
        btnPressed();
        //       btnDown = (Btn || btn1 || btn4); old backwards support of 'Btn'
        btnDown = (btn1 || btn4);
        if (btnDown && !inAir) {
          if (BtnPressed) {
            dimFlash = 0;
            dribbleCnt++;
            if (ver == 1 && shot == 3 && dribbleCnt > 1) { // coming in for a lay-up
              spot++;
              if (lastLCnt > 0) thrust += 40000 / (loopCnt - lastLCnt); // add momentum to thrust
              lastLCnt = loopCnt;
            }
          }

          if (dribbleCnt < 3 || shot == 3) {
            brightOne = spot;
            if (BtnPressed) beepTone(170,50);  // bounce the ball
          }
          if (dribbleCnt == 3) thrust++;
        } else {
          brightOne = 0;
        }

        released = (!btnDown) && BtnChanged;
        if (released && dribbleCnt >= 3) { // the ball takes flight
          inAir = true;
          //          Serial.print("thrust: "); Serial.println(thrust);
          //         rate=(thrust+1000)/20;  //for thrust=1000 gives 100 which results in 10 spots in one second
          rate = (thrust + 750) / 15; //for thrust=7500 gives 100 which results in 10 spots in one second
        }

        if (inAir && (loopCnt % 200) == 0) {
          if (thrust <= 30) { // end of ball flight
            refreshWait(150);
            if (litLED == nleds - 1) {
              winner(1);
              playing = true; // reset, due to winner()
              score++;
              //   continue; // seems to be a bug in compiler, this went to the else clause, Boo-Hiss
            } else { //missed
              beep(20);
              pigCnt++;
            }
            turn = 0;
            refreshWait(500);
            continue;
          }

          travel = travel + rate;
          thrust -= rate;

          litLED = spot + travel / 100;
          if (litLED > nleds) litLED = (2 * nleds) - litLED;
          if (litLED < 1) thrust = 0;
        }

        if (pigCnt >= 3 && ver == 1) { // =3 for PIG  would-be  =5 for HORSE
          showColor(5); refreshWait(100);
          clearDisp();  refreshWait(600);
          if (score > 0) {
            boOop(50);
            showLevel(score);
            refreshWait(2000);
          }
          else buZZ(150, 500); // & with no baskets made!
          playing = false;
          break;  // from this turn
        }

        //================ UPDATE Display
        loopCnt++;
        if (dribbleCnt == 0) {
          dimFlash = spot;
          if (ver == 2 && btn2Pressed) {
            if (--spot < 1) spot = 1;
            shot = 0;
          } else if (ver == 2 && btn3Pressed) {
            if (++spot > nleds - 2) spot = nleds - 2;
            shot = 0;
          }
          dimFlash = spot;
        }

        refreshDisp();
        delay(1);

        // if 'esc' button held down or Fnc-1 ... Exit
        if (ESC || (btn4 && btn1)) {
          delay(300);
          return;
        }
      }
    }
  }
}

void Joust() { // When your foe is upon you strike him down
  // ====================================     Shoot Out
  //             When he's in your sights ... fire!, Competitively with two players
  // Challenge...
  // hit with btn2, hold till all dim, switch to btn3, hold till all are off, & release leaving no dims
  byte hits1, hits4;
  bool faceoff, flip, doingChallenge;
  byte speed;

  spriteOne = 1;
  spriteTwo = nleds;
  loopCnt = 0;
  hits1 = hits4 = 0;
  faceoff = flip = false;
  doingChallenge = false;
  speed = 1;

  while (1) {
    loopCnt++;
    refreshWait(1);
    scanBtns();
    if (ESC && !btn1) break;

    if (loopCnt % (600 - speed * 125) == 0) {   // consider influence over time, at a given cadence
      spriteOne++; // ++ go right
      spriteTwo--; // -- go left
      if (spriteOne > nleds)  spriteOne = 0;
      if (spriteTwo < 0)      spriteTwo = nleds;
      if (spriteOne == 0 && random(4) != 0) spriteOne = 1; // 1/4 of the time Skip!
      if (spriteTwo == 0 && random(4) != 0) spriteTwo = nleds;
      // fidget support ..
      //          if (btn2) dim[spriteOne] = (flip)? !dim[spriteOne] : 0;
      //          if (btn3) dim[spriteTwo] = (flip)? !dim[spriteTwo] : 1;
      if (btn2 && flip) dim[spriteOne] = !dim[spriteOne];
      if (btn3 && flip) dim[spriteTwo] = !dim[spriteTwo];
    }

    if (spriteTwo == spriteOne || spriteTwo == spriteOne + 1) { // opponents are face-to-face or on the same spot
      faceoff = true;
      if (btn1Pressed && !btn4) {
        spriteTwo = 1;
        beep(120);
        hits1++;
      } else if (btn4Pressed && !btn1) {
        spriteTwo = nleds;
        beep(120);
        hits4++;
      }
      if (hits1 == 3 || hits4 == 3) { // achieved the Wining # of hits
        beep(60); beep(60); beep(60); refreshWait(500);
        winner(hits1 == 3);
        refreshWait(1000);
        break;
      }
    } else {    // check to penalize miss placed shots
      faceoff = false;
      if (btn1Pressed && hits1 > 0) hits1--;
      if (btn4Pressed && hits4 > 0) hits4--;
    }

    // support fidget challenge to transform background
    if (btn1Pressed) doingChallenge = false;
    if (btn2Pressed || btn3Pressed) doingChallenge = true;
    if (btn2Pressed && faceoff) flip = !flip; // set flip=true only if it is not already
    if (btn3Pressed && faceoff) flip = !flip; // so must release one before the other can enable it
    if (!(btn2 || btn3)) flip = false;
    if (btn4Pressed && doingChallenge) {
      for (int i = 1; i <= nleds; i++) dim[i] = 0; // clear background
      speed = (speed + 1) % 5; // shift speed
    }
  }
}

// =====================================================================
void UpDownSide() { // Competitively land balls in indicated spots to dominate
  // land a ball in each posible "target" spot to flip it
  // to Up(on) for the Left side or Down (0ff) for the right side.
  // hold Btn1 (player1) / Btn4 (player 2) to swing, and let go to throw
  int player = 1;
  int i, cnt, r;
  byte spot, score, btn, target;
  int travel, rate, thrust, dTrv;
  bool rolling, newPressed, released, btnDown;
  byte ver, pass;
  bool setOn;
  bool turn;
  byte missed[2 + 1];

  ver = getOption(2);
  if (ver == 0) return;
  level = 1;
  missed[1] = 0; missed[2] = 0;

  clearDisp();
  refreshWait(500);

  while (1) {
    // setup game-level
    wait_BtnRelease();   // wait for no button pressed
    gameID++;
    score = 0;
    loopCnt = 0;
    clearDisp();
    // //for (i=1; i<=nleds; i++) dim[i]=i%2;  // lite every other
    // //    dim[r=random(nleds/2)+1]=1;   dim[r+nleds/2]=1;  // lite random 4 symeterically
    // //    dim[r=(r+random(nleds/2-1))%(nleds/2)+1]=1;   dim[r+nleds/2]=1;
    // dim[r = random(nleds / 2) + 1] = 1;    // lite random 2 of 4
    // dim[r = (r + random(nleds / 2 - 1)) % (nleds / 2) + 1] = 1;
    // if (random(2)) {
    //   dim[5] = dim[1];  // repeat on other side
    //   dim[6] = dim[2];
    //   dim[7] = dim[3];
    //   dim[8] = dim[4];
    // } else {
    //   dim[5] = dim[4];  // reflect "
    //   dim[6] = dim[3];
    //   dim[7] = dim[2];
    //   dim[8] = dim[1];
    // }
    for (i=0; i<nleds/2; i++) dim[PRIG(i,gameID,nleds)+1] = 1;  // lite a random set of 1/2 the LEDs

    playing = true;
    while (playing) {
      refreshWait(600);
      spot = (player == 1) ? 1 : 8;;    // where we are throwing from
      btn = (player == 1) ? 1 : 4;
      r = random(nleds);
      for (i = 1; i <= nleds; i++) { // find a target for this player
        target = (i + r) % nleds + 1;
        if (dim[target] == (player == 2)) break;
      }
      if (missed[player] && random(2)) target = missed[player]; // 1/2 the time repeat a missed target
      missed[player] = 0;
      travel = 0;
      thrust = 0;
      rolling = false;
      turn = true;   // turn in progress
      while (turn) {
        bool wasDown = btnDown;
        bool pressed = btnPressed();

        if (!rolling) {
          lit[spot] = (loopCnt % 1250) > 800 && (loopCnt % 1250) < 860;
          lit[target] = (loopCnt % 1250) > 1000;
          if (target == spot && (loopCnt % 1250) > 1200) lit[target] = 0; // reduce flash where needed
        }
        btnDown = (btnNum == btn);
        if (ESC && (player == 1 || btn1)) return; // uses ESC while its P1's turn (or b1+b4)
        if (btnDown && !rolling) {
          thrust++;
          spriteOne = spot;
        } else {
          spriteOne = 0;
        }

        released = (!btnDown) && wasDown && BtnChanged;
        if (released) {       // the ball takes flight
          rolling = true;
          lit[spot] = false;
          lit[target] = false;
          //Serial.print("thrust: "); Serial.println(thrust);
          rate = (thrust + 750) / 15; //for thrust=7500 gives 100 which results in 10 spots in one second
          rate = rate / 4; // patch so we can update more often for smoother fast motion balls
        }

        if (rolling && (loopCnt % 50) == 0) { // was %200 before patch above

          if (thrust <= 30) {         // coming to a stop
            //Serial.print(" pass "); Serial.print(pass);
            //Serial.print("   LED "); Serial.println(litLED);
            if (target == litLED && ver == pass) { // they hit it !
              setOn = !dim[litLED];
              dim[litLED] = setOn;
              if (setOn) beep(70);
              else bop(90);
            } else {
              missed[player] = target;
              buZZ(20);
            }

            cnt = 0;
            for (i = 1; i <= nleds; i++) cnt += dim[i];
            if (cnt == 0 || cnt == nleds) { // game over
              winner(cnt != 0);
              playing = false; // reset, due to winner()
            }

            turn = false;
            refreshWait(1000);
            litLED = 0;
            continue;
          }

          travel = travel + rate;
          thrust -= rate;
          dTrv = travel / 100;
          spot = (dTrv % nleds) + 1;
          if (player != 1) spot = (nleds + 1) - spot;
          litLED = spot;

          pass = 1;
          if (dTrv >= nleds)  pass = 2;
        }

        //================ UPDATE Display
        loopCnt++;
        if (spot == target && !rolling) loopCnt++; // extra flash where needed

        refreshWait(1);
        if (btn1 && btn4) {
          return; // both to Exit
        }
      }
      player = 3 - player; // toggles 1,2,1,2 ...
    }
  }
}

// ***************************  Group 3  *******************************
// ================= Spirted Fun and Psyche-Ometers ===========================

void Whack_Mole() { // see how many Moles you can Whack in 30 seconds
  byte i, rnum;
  byte mloc, numMoles;
  byte pinState = 0;
  int hitCnt, wait4it;
  int maxCnt;
  int need, twos2go, threes2go;
  bool hit;
  unsigned int runID;
 #define MOLE colorLeds

  ver = 1;  // standard real world behavior; there is only one mallet and it can only hit one mole at a time
  // an idea for a version #2 would be to allow simutaeous multiple button pusses, you could illiminate sets of moles faster.

  clearDisp();
  clearDisp(); refreshWait(500);
  ledsAll(15); refreshWait(500);    // indicates we have selected the game
  clearDisp(); refreshWait(1500);
  if (ESC) {
    refreshWait(300);
    return;
  }
  t0 = millis();
  hitCnt = 0;
  maxCnt = 1600;  // how long a set of moles are exposed (in wouldbe millisecs-ish)
  need = 0;
  refreshWait(1);
  while ((millis() - t0) < (30 * 1000)) {  // is this time period up ?
    digitalWrite(ledOnBrd, mySecs & 1); // DEBUG flash
    //Serial.print("miliSec Count: ");  Serial.println(millis() - t0);
    if (maxCnt > 400) maxCnt -= 40;  // decreases mole exposure as game goes on
    loopCnt = maxCnt;
    for (i = 0; i <= 4; i++) MOLE[i] = 0;

    if (need == 0) {
      need = nleds; // -------  need a set of nleds 'random' #'s with distribution of ~ 50% 1s, 33% 2s, and 17% 3s.
      twos2go = 4;
      threes2go = 2;
    }
    rnum = random(0, need);
    if (rnum < threes2go) {
      threes2go--;
      numMoles = 3;
    } else if (rnum < (twos2go + threes2go)) {
      twos2go--;
      numMoles = 2;
    } else {
      numMoles = 1;
    }
    need--;

    // ------------- here was utilizing the random function to select mole locations
    // mloc = random(1, 5);
    // MOLE[mloc] = 1;
    // if (numMoles > 1) {
    //   mloc = ((mloc + random(0, 3)) % 4) + 1;
    //   MOLE[mloc] = 1;  // add in second mole
    //   if (numMoles > 2) { // add in a third mole
    //     mloc = ((mloc + random(0, 2)) % 4) + 1;
    //     if (MOLE[mloc] == 1) mloc = (mloc % 4) + 1;
    //     MOLE[mloc] = 1;
    //   }
    // }
    // ------------- now instead use PRIG function to select mole locations
    runID=random(999999);
    for (i=0; i<numMoles; i++)  MOLE[ PRIG(i,runID,4)+1 ] = 1;

    dspColors();
    wait4it = random(50, 250); // preset wait time between sets of moles
    while (0 < loopCnt--) {
      //      dspColors();
      refreshWait(1);
      scanBtns();
      hit = false;
      if ((btn1 + btn2 + btn3 + btn4) == 1 || ver == 2) {
        if (btn1 && MOLE[1]) {
          MOLE[1] = 0;
          hit = true;
        }
        if (btn2 && MOLE[2]) {
          MOLE[2] = 0;
          hit = true;
        }
        if (btn3 && MOLE[3]) {
          MOLE[3] = 0;
          hit = true;
        }
        if (btn4 && MOLE[4]) {
          MOLE[4] = 0;
          hit = true;
        }
        pinState = 1 - pinState;
        if (!hit && ver != 2) digitalWrite (BEEPPIN, pinState); // make noise if missing, without causing delays
      } else {
        digitalWrite (BEEPPIN, LOW);
      }
      dspColors();
      if (hit) {
        hitCnt++;
        boop(16); // mallet hitting mole
        if ((MOLE[1] + MOLE[2] + MOLE[3] + MOLE[4]) == 0) {
          wait4it = 20;
          break;
        }
        if (ver == 1) wait_BtnRelease(); // to insure no double btn presses are taken
      }
      if (ESC && !btn1) {
        refreshWait(300);
        return;
      }
    }
    if (ESC && !btn1) {
      refreshWait(300);
      return;
    }
    digitalWrite (BEEPPIN, LOW);
    refreshWait (wait4it);
    wait_BtnRelease();  // must have mallet off the moles before proceding
  }
  //Serial.print("max Count time allowed: ");  Serial.println(maxCnt);
  //Serial.print("Hit Count: ");  Serial.println(hitCnt);
  // ----------------  Show score. Show 1 led for each 10 hits
  showLevel(hitCnt / 10);
  refreshWait(2000);
  while (!btnPressed());
}

//----------------------------------------------------------------------
void Speed_Scan_r4() {  // hit streaking LED on the right spot to speed things up to a winning max
  // misses slows things down, and can put you even further from winning
  int onMSecs, stRate = 150;
  int hits = 0, goal = 9;

  litLED = 1;
  onMSecs = stRate;
  while (true) {
    refreshWait(onMSecs);   // light one LED (moving blip) for a bit
    //btnsNow();   // get a definitive state update (despite debounce)
    stableBtns();  // get a full state once btns states are stable
    // allow use of left or right most button
    if (BtnPressed && (litLED == 5 || litLED == 6)) {
      beep(100);
      hits++;
      if (hits == goal) {
        beep(100, 50); beep(100, 50); beep(250); // you did it
        hits = 0; // start over
        refreshWait(1000);
        clearDisp();
      }
    }

    // added for support of a missed penalty
    if (BtnPressed && !(litLED == 5 || litLED == 6)) {
      refreshWait(300);
      hits--;     // slow down update rate
      clearDisp();
    }

    onMSecs = stRate - (15 * hits); // more hits he get faster the blip scans
    for (int i = 1; i <= hits; i++) dim[i] = true; // display as, dim, the hit progress
    litLED = (litLED % nleds) + 1;    // advance to next LED
    if (ESC) break;
  }
}

void KaBoom() {   // inspired by Atari 2600 KaBoom! game  https://www.youtube.com/watch?v=_v9D_vDBE40 
  int waitMS=290; //stRate = 300;
  int cnt=0, score=0;
  int bmove[5]={0,-2,-1,1,2};
  int katch=2, kaB;
  int drop=2, dir=1, dircnt=0;
  bool play=false;

  int ver = getOption(2);  // ver 1: you should catch the falling bombs   ver 2: avoid all bombs
  while (!ESC) {
    DispScanWait(waitMS/2); // wait MS time, while updating display and capturing any btn press
    if (BtnPressed) {katch=katch + bmove[btnNum] ; play=true;}
    DispScanWait(waitMS/2); // now, allowing 2 btn press detections per play update
    
    katch=katch + bmove[btnNum] ;
    if (katch<1) katch=nleds-katch;  // wrap-around as needed
    if (katch>nleds) katch=katch-nleds;
    if (BtnPressed) play=true;
    flashLed=katch;
    blinkLed=katch;

    if (drop<1) drop=nleds-drop;  // wrap-around as needed
    if (drop>nleds) drop=drop-nleds;
    kaB=brightOne;
    brightOne=litLED; // --- update falling bombs
    litLED=dimLED;
    drop += dir;
    dimLED=drop;
    dircnt++;
    if (dircnt>3 && random(4)==0) { // --- recalc direction
      dir= -dir; // reverse dir at some point after 3 in a row
      dircnt=0;
    }

    if (play) {
      if (ver==1 && cnt>2) {
        if (blinkLed==brightOne || blinkLed==kaB) { // caught/disarmed a bomb in time
          score++;  // should be maximized
        } else {
          buZZ(45);
        }
      } else {  // ver == 2
        if (blinkLed==brightOne || blinkLed==litLED || blinkLed==dimLED) { // set off a bomb
          score++;  // should be minimized
          buZZ(45);
        }
      }

      if (++cnt>=35) {   // attack of 32 incoming is complete
        showLevel(score);
        refreshWait(3000);
        clearDisp();
        cnt=0, score=0;  // then go again
        drop=katch;
        play=false;
      }
    }
  }
}

  // ========================================    Telekinesis
void Telekinesis() {  // LED wanders due to environmental sampling. Challenge your predictive powers.
  // uses ADC input (ADC_b), and works best with 2+ inches of wire attached to it
  unsigned int x, q;
  int pos, lastPos;
  bool btnOccured;
  int n = 0, cnt = 0, cntMax = 0, pass=0;
  int dir[5];
  int maxDir, maxSum;
  unsigned int sum;
  int tst[5];
  bool pause;

  //debugPrt=true;
  ver = get_Mode(2); // 1: normal   2: more assured balanced behavior/swings
  if (ver == 0) return;

  pos = nleds / 2;
  goal = nleds;
  clearDisp();
  btnOccured = false;
  while (1) {
    pass = ++pass %1001;
    cnt = sum = maxSum = 0;
    tst[0] = tst[1] = tst[2] = tst[3] = tst[4] = 0;

    for (int i = 0; i < 5; i++) dir[i] = 0;
    while (cnt++ < 25) {    // consider influence over time
 
      x = analogRead(ADC_b); // read only every 2-5 msecs  (A0|A6 now A7 its available on ProMicro)
      //        Serial.print("Analog Val:  "); Serial.println(x);  // TESTing
      refreshWait(random(20));
      x += analogRead(SENSOR); // patch for greater reading
      refreshWait(10);
    //#ifndef RP2040
    //      x += analogRead(A6);  // Nano works better with more randomity input
    //#endif

      if (ver == 2) x += pass;  // adj for greater balanced results (& harder too sway)
      else x+=cnt;
      sum += (x % 256); // statistics keeping
      // half the time stay put & 1/2 the time go right or left
      q = x%5; // we only tract for 4 (quadrants) 0-3 and ignor the odd fifth one out
      tst[q]++;
      dir[q]++;
      if (q!=4 && dir[q] > maxSum) {
        maxSum = dir[q];
        maxDir = q;
      }
      if (btn4Pressed) pause = true;
      scanBtns();
      btnOccured = btnOccured || (BtnPressed && !btn4Pressed) ;
    }
    if (debugPrt) {
      Serial.print("Mean: "); Serial.println(sum / 100);
      Serial.print("  4bit Cnts: "); Serial.print(tst[0]); Serial.print(" "); Serial.print(tst[1]); Serial.print(" ");
      Serial.print(tst[2]); Serial.print(" "); Serial.print(tst[3]);
      if (maxDir == 0) Serial.print("   >");      // go right
      else if (maxDir == 3) Serial.print("  <"); // go left
      Serial.println();
    }

    while (btn4Pressed || pause) {  // implement pause func
      spriteOne=0;
      if (btn4Pressed) pause = !pause;  //toogle in&out of pause
      if (!pause) {
        setColor(5);  // announce resumption
        beep(30);
        setColor(0);
      }
      scanBtns(); refreshWait(1);
      if (ESC) return;
    }

    sum = 0;
    lastPos = pos;
    if (maxDir == 0) pos++;      // go right
    else if (maxDir == 3) pos--; // go left
    if (pos < 0) pos = nleds - 1;
    pos = pos % nleds;
    spriteOne = pos + 1;
    if (lastPos != pos) {
      if (btnOccured) dim[pos + 1] = !dim[pos + 1]; // if predicted invert the Led we are now at
      btnOccured = false;
      n = 0;
      for (int i = 1; i <= nleds; i++) n += dim[i];
      if (n == goal) { // if they have set or cleared them all congradulate them
        taDah();
        goal = (goal == 0) ? nleds : 0;
      }
    }
    if (ESC) break;
    secsIdle=0; // disable idle timeout
  }
}

// ======================== ghostbuster's type P.K.E. meter
void PKE_Meter() {  // ----- psycho-kinetic scope Detecting Psychokinetic/ParaNormal Energy
  float baseline;   //a small % value (e.g. 10) used to scale RNG rating
  float PKE_level;  // displayed scaled level of PKE
  // Mathematically normal quiescent values:
  float runningMean = 127.5, runningGMean = 113.2, runningDelta = 85.33, runningBChg = 4.0;
  float runningDcnt = 1.50;  // direction cnt, # of consecutive increases or decreases 
  float runningMS=0.0, runningRMS=1.0, runningChgV=1.0;
  float rateRNG, lastRNG;  // rating RNG randomness (0.0=True Random,  1.0=totaly Not random)
  float deltaSum;
  byte chgd, st;
  unsigned int x, lastX = 127;
  int seqDelta, geoMean;
  long bitChgSum = 0;
  bool reading;  // true while taking readings
  int cnt;
  unsigned int px=127,lx=127,hx=127;
  unsigned int prt;
	int d_cnt=0, d_run, dir;

  ver = get_Mode(2); // 1: linear  2: logrithmic output display
  if (ver == 0) return;
  // start off with a display sweep to Max level & back to level 1
  for (int i = 1; i <= nleds - 1; i++) {
    dispPKE(i);
    refreshWait(120);
  }
  for (int i = nleds - 1; i >= 1; i--) {
    dispPKE(i);
    refreshWait(120);
  }

  // set level 1 = to a standard minimal % of non randomness
  baseline = 12.0 / 100.0; // set empirical (w/ 12 LEDS I used 10%?)
  lastRNG = baseline;
  //  Serial.print("  Non-Randomn baseline: "); Serial.println(100 * baseline);

  // sweep from max to level 1 at start
  beep(40);
  clearDisp();
  reading = true;
  while (reading) {
    // Make Analog readings to generate a random value 0-256
    // using HRNG(2) which is about twice as solidly random as is HRNG(1),straigt ADC in.
    // and therefore less sensitive to a false NRP. I think HRNG(1) has too little signal to noise.
    // in Mega_Game proj. HRNG(2)'s randomness is software amplifies as there are no diodes on Ain
    //x = HRNG(2) & 0xFF; // take an 8bit reading, was using HRNG(3)
    x = (HRNG(2)/2) & 0xFF;  // take middle 8 of 10 bits
    // HRNG(0) gives 'random()' #s with ~ constant randomness thus ~ 0 display output
    
    for (int i=0; i<25; i=i+5) { // sampling rate, has stabiity side effects (is evey 25 ms)
      refreshWait(5);
      if (PKE_level>4 && random(3)<(PKE_level-4)) digitalWrite (AUDIO_OUT, random(2));  // more reading noise
    }
    
    // smooth out raw input a little ... Maybe too much smoothing for this app
    // if (x>px) hx=x;
    // else lx=x;
    // px=x; // average with privious high or low
    // x=(lx+hx)/2;

    //x=(x+px)/2; px=x;

    // Determin a running randomness rate
    chgd = x ^ lastX;
    bitChgSum = 0;
    for (int i = 0; i < 8; i++) {
      bitChgSum += (1 & (chgd >> i));
    }
    seqDelta = abs((int)lastX - (int)x);
    geoMean = sqrt(x * lastX);  //   geometric mean

		// below gives a local randomness measurement resulting in 1.500 (for true random)
    // tracking the number of consecutive times 'X' continues in the same direction (need a good name for it)
		d_cnt++;
		if ((x > lastX) != dir) {
			dir = (x > lastX);
			d_run = d_cnt;
			d_cnt = 0;
		} else {d_run = 0;}

    lastX = x;

    // calculate a Change Velocity
    deltaSum = abs(x-runningMean)/127.5;
    deltaSum += abs(geoMean-runningGMean)/113.2;
    deltaSum += abs(seqDelta-runningDelta)/85.3;
    deltaSum += abs(bitChgSum-runningBChg)/4.0;
    deltaSum += abs(sqrt(runningMS)-runningRMS)/50.0;
    deltaSum = deltaSum/5;
    runningChgV = (9.0 * runningChgV + deltaSum) / 10.0;

    // track these values used to evaluate randomness
    runningMean  = (9.0 * runningMean + x) / 10.0;
    runningGMean  = (9.0 * runningGMean + geoMean) / 10.0;
    runningDelta = (9.0 * runningDelta + seqDelta) / 10.0;
    runningBChg  = (9.0 * runningBChg + bitChgSum) / 10.0;
    if (d_run)   runningDcnt  = (9.0 * runningDcnt + d_run) / 10.0;

    runningMS  = (9.0 * runningMS + (runningMean-x)*(runningMean-x) ) / 10.0; // runningMean repesents the DC offset of an AC signal
    runningRMS = (9.0 * runningRMS + sqrt(runningMS))  / 10.0;;

    // combine their deviations from their Norms
    rateRNG  = abs((runningMean / 127.5) - 1.0);
    rateRNG += abs((runningGMean / 113.2) - 1.0);
    //rateRNG += abs((runningDelta / 85.33) - 1.0);
    rateRNG += abs((runningRMS / 50.) - 1.0);
    rateRNG += abs((runningBChg / 4.0) - 1.0);
    rateRNG += abs((runningDcnt / 1.5) - 1.0);
    rateRNG = rateRNG / 5; // average them. expected range 0.0 - 1.0

    #ifdef RP2040
      rateRNG = rateRNG / 2;  // normalize with Nano results
    #endif  
    // rateRNG = (rateRNG + 3*lastRNG) /4; // smooth it out a little
    rateRNG = (rateRNG + 7 * lastRNG) / 8; // smooth it out a little more
    lastRNG = rateRNG;
    //        if (debugPrt && (cnt++ % 20)==0) {  // print Randomness evaluation numbers
    //          Serial.print("  mean: "); Serial.print(runningMean);
    //          Serial.print("  GeoMean: "); Serial.print(runningGMean);
    //          Serial.print("  Seq.Delta: "); Serial.print(runningDelta);
    //          Serial.print("  Avg Bits Chg'd: "); Serial.print(runningBChg);
    //          Serial.print("  Non-Randomn Percent: "); Serial.println(100 * rateRNG);
    //        }

    // if ((myMillis / 100) != prt) {
    //   prt =(myMillis / 100);
    //   Serial.print(runningMean); Serial.print(" "); Serial.print(runningGMean); Serial.print(" "); Serial.print(runningDelta); 
    //   Serial.print("   "); Serial.print(runningBChg); Serial.print(" "); Serial.print(runningDcnt);
    //   Serial.print(" ");  Serial.print(runningRMS);
    //   Serial.print("   Non-Randomn (sig.) Percent: "); Serial.print(100 * rateRNG);
    //   Serial.print("   Chg. Velocity: "); Serial.print(100 * runningChgV);
    //    Serial.print("   \tADC SENSOR, a & b: "); 
    //    Serial.print(analogRead(SENSOR)); Serial.print("  "); Serial.print(analogRead(ADC_a)); Serial.print("  "); Serial.println(analogRead(ADC_b));
    // }

    rateRNG = (rateRNG + runningChgV)/2;  // now combine rateRNG level with the Change velocity for more responsive reading
    PKE_level = rateRNG / baseline;
    if (ver == 2) {
      //            given logb(x) = ln(x)/ln(b)
      //PKE_level = log(PKE_level)/log(1.5);  // was 1.4, went to 1.5 to reduce sensitivity
      PKE_level = (nleds + 2) - log(100 * (1.0 - rateRNG)) / log(1.6); // ~ a reverse orientation of a log scale
    }

    dispPKE(PKE_level);
    if (PKE_level>1 && random(3)<(PKE_level-1)) { // make signal noise proportional to PKE_level
      digitalWrite (AUDIO_OUT, st=random(2));
      // if (Loud && st) { // TBD add Loud support here
      //   refreshWait(1);
      //   digitalWrite(AUDIO_OUT, 0);
      //   digitalWrite(AUDIO_NEG, 1);
      // }

      refreshWait(1);
      digitalWrite (AUDIO_OUT, random(2));
      //digitalWrite (AUDIO_NEG, 0);
    }
    
    scanBtns();
    if (btn1Pressed && ver == 1) { // recalabrate
      wait_BtnRelease(); // wait for no button
      //Serial.print("  Non-Random Percent: "); Serial.println(100 * rateRNG);
      //          baseline = (baseline+rateRNG)/2;  // set level 1 halfway to current level
      baseline = rateRNG / 2; // set current level to be = display level 2
      //Serial.print("  new baseline: "); Serial.println(100*baseline);
    }
    //if (btn3Pressed) Loud=!Loud;
    if (btn4) return;
    secsIdle=0; // disable idle timeout
  }
}

// ----------- Hardware (Real/Quantum World) based Random Number Generator  -----------------
int HRNG(byte algo) {    // --- used by PKE_Meter to gather evidence of PKE signals ---
  // 2021 update 3 -> 4 algos, with new #2 with Ain reading being Enhanced by a second reading
  int sensorValue;
  byte y, yRev, s;  // use of a byte restricts results to 8 bits (0-255)
  char c;
  unsigned int x;
// #define DETECTOR SENSOR  // not sure if 2 pass defines will work

  // read ADC value
  // with the next line's analog read of a line with a pull-up on it, the DETECTOR line's Randomness is:
  //   Non-Randomn Percent: ~40,     while w/o it is:  ~15
  sensorValue = analogRead(DETECTOR);

  switch (algo) {
    case 0:   // ------------  use the random function  ---------------
      //  calculated Pi: 3.15   mean: 127.12   Avg Bits Chg'd: 4.00  dist: 1.07    Random Mark missed by: 0.95 %   Running avg Err: 0.94 %
      x = random(256);
      break;

    case 1:   // Use a direct Analog Read
      // calculated Pi: 3.00   mean: 130.35   Avg Bits Chg'd : 3.09  dist: 1.19    Random Mark missed by: 20.23 %   Running avg Err: 20.88 %
      x = sensorValue;  // direct Analog Read (use lower bits 8 as above, or 10 of them)
      break;

    case 2: // Amplify Ain randomness with another read, offset
      x = sensorValue;  // direct Analog Read (uses lower 8 bits) mean:140  geoMean:90  seq.D:120 avgBitsChgd:4.2  NRP:16  erratic
      // w/o external back biased diode pair attached to the Ain, following is used to amplify the randomness
      //x = (analogRead(ADC_b)+sensorValue)/2; // its quiescent levels: ~ mean:120  geoMean:115  seq.D:65  avgBitsChgd:3.3  NRP:12
//      x = (x << 4) ^ analogRead(SENSOR);
      //x = (x << 4) ^ analogRead(DETECTOR);    // ~ same #s but using only one pin input
x = (x+ analogRead(SENSOR));
      break;

    case 3:  // reverse read bit order and add in 2 more reads
      // with + length of wire looks close to as good as case #0
      // w/ 2 readings, Pi: 3.12  mean: 127.70  Avg Bits Chg'd: 3.91  dist: 1.06    Random Mark missed by: 2.65 %  Running avg Err: 2.96 %
    case 4:  // Combine this Hardware RNG value and Pseudo SW RandomNumber
      y = sensorValue;
      //          reverse the bits
      yRev = 0;
      for (int i = 0; i <= 7; i++) {
        yRev = yRev << 1;
        yRev += y % 2; y = y >> 1;
      }
      sensorValue = analogRead(DETECTOR);
      // with double reading Avg Bits Chg'd: 3.44 -> 3.65
      sensorValue = sensorValue + analogRead(DETECTOR);  // ' '  &  improved err (-2%)
      y = yRev + sensorValue;    // Avg Bits Chg'd: 3.44   w/no wire, to 4.0 with wire
      x = y; // the byte 'y' kept results to 8 bits
      break;
  }

  if (algo == 4) { // use for best of both worlds
    y = y + random(256);
    x = y;
  }

  return (x % 1024); // x is an Integer, but in most cases only has 8 bits in it (& not >1023)
}

// --------------------------------------------
void dispPKE(byte level) {  // utility to display a representation of the detected PKE levels
  byte midLED;

  if (level >= nleds) {
    level = nleds;  // over-scale makes clicking sound
    beep(4);
  }
  clearDisp();
  midLED = nleds / 2;
  if (level == 0) {
    dim[midLED] = dim[midLED + 1] = true;
    return;
  }
  // set bright the level LEDs
  if (level <= midLED) {
    spriteOne = midLED + level;
    spriteTwo = midLED + 1 - level;
  } else {  // level>midLED overflows and form a trail of dim leds
    spriteOne = nleds - (level - midLED);
    spriteTwo = level - (midLED - 1);
    for (int i = spriteOne; i <= nleds; i++) dim[i] = true;
    for (int i = 1; i <= spriteTwo; i++) dim[i] = true;
  }
}

// ---------------------------------------------------------------------
void Affinity_Meter() {  // implements a Temperament / Affinity / Strength / Love Meter ...
  // Testee holds connect to ADC_a in one hand and grnd in the other. (internal pull-up is used)
  byte lSide, rSide;
  const float runningXMean = 512; // for running mead of read Analog value (range 0-1023);
  float ravgX;
  int lastLED;
  int x;
  float sumX;
  int n, cnt, cntMax;
  bool ackd, t;
  //  const int xvals[] = {0,26,75,119,178,246,375,525,646,742,805,853,889,1024}; // used with 12 LEDs
  // for use with 8 LEDs
  const int xvals[] =   {0, 80, 150, 220, 290, 390, 525, 750, 32000}; // focus on usefull range. w/40K pullup
  // Affinity reading of 2-4 (a person's net impedence 5-12K) are just right, not too low & not up tight .

 #ifdef RP2040
 #ifdef DIO
  pinMode(DIO, OUTPUT);   // provide a GRND reference for use of adjasent pins A2-D0 on Ext conn 
  digitalWrite(DIO, LOW); // also done in loop()
 #endif
 #endif

  pinMode(SENSOR, INPUT_PULLUP);
  pinMode(ADC_a, INPUT_PULLUP); // with RP2040 only _a supports the pull-up, may need to connect _a & _b for this code to work
  pinMode(ADC_b, INPUT_PULLUP); // Nano: 40K to Vcc (3.6-5v)  RP2040: 50K-80K to 3.3v
  // with RP2040 there seems to be cross effects with A2/A3 pull up operations, making it hard to predict what you'll get

 
 //debugPrt=true;  // TBD TESTing

  while (1) {
    n = cnt = cntMax = 0;
    ackd = false;
    clearDisp();
    beep(25);
    for (int i = 1; i <= nleds; i++) dim[i] = true;
    refreshWait(1000);
    lSide = 0; rSide = nleds;
    ravgX = runningXMean;
    litLED = ravgX / 100 + 2; // put it in the ball park

    while (1) {
      if (lSide <= rSide) { // initially sweep Leds in on the 'x' pos
        if ((litLED - lSide) > (rSide - litLED)) dim[lSide++] = false;
        else dim[rSide--] = false;
        if (lSide <= rSide) litLED = 0;
        sumX = ravgX;
        n = 1;      // delays locking an average in
        cnt = 0;
      } else {
        if (cnt > 4 && !ackd) { // tell em that's it
          ackd = true;
          beep(50, 50); beep(50);
        }
      }
      for (int i = 0; i < 10; i++) { // get trending value
        brightOne = litLED;
        DispScanWait(50);  // meanwhile upd display & preserve button presses
        if (btn1 || btn4) break;
        x = analogRead(SENSOR); // reads ADC for inviro value 0-1023
 #ifdef RP2040_PRO  // needed for SparkFun ProMicro RP2040s 
       x = x >>2; // reduce to 10 bits (from 12)
       x = x + (x/5); // scale up 20% due to 50K pullup as opposed to 40K
 #endif
 #ifdef RP2040
       x = x + (x/5); // scale up 20% due to 50K pullup as opposed to 40K
 #endif

        {Serial.print("  X: "); Serial.print(x);}
        if (ravgX == 0.0) ravgX = x;
        ravgX = (5.0 * ravgX + x) / 6.0;
      }
      sumX += ravgX;
      n++;
      x = sumX / n;
      //if (debugPrt)
             {Serial.print(" sumX/n = X: "); Serial.println(x);}

      litLED = 0;
      while (xvals[++litLED] < x) {}  // show current tracked value
      cnt = (litLED == lastLED) ? cnt + 1 : 0; // cnt the settling
      lastLED = litLED;

      if (debugPrt)       {Serial.print(" ravgX: "); Serial.print(ravgX);}
      //        scanBtns();  // scanned above
      if (btn4) return;
      if (btn1) break;   // go again
    }
  }
}

// ==========================================  Conjure like a Magic-8-Ball or Fortune teller
// Magic-8-Ball: hold down btn1 ... ponder question, let go.  Red: NO,  YEL: Maybe,  Grn: YES  Blue: Try again
// use btn2 for 1/6 out comes ~ like a dice, useful for determining one's fortune or options in games.
// use btn3 for 1/8 could be used like Tarot or Horoscope card telling or a Game/Fortune wheel
//
// ideas: Sound of dice rattling could be added while B2 held; after b3: the sound of fortune wheel clicker as it rolls to stop.
void Fortune_Teller() {  // Simulate a Magic-8 ball, rolling of a Die, and a Wheel of Fortune
  int cnt, maxCnt;
  int mySecs;
  byte ans, num, keyCode;
  byte priorBtn;
  static byte dicePat[] = {0x10, 0x42, 0x2A, 0xA5, 0xD3, 0xE7};
  int steps, st;
  while (1) {
    clearDisp();
    while (!Btn) {
      scanBtns();
      mySecs = msCnt / 1000;
      litLED = mySecs % 2; // toggles 0/1 per second
      if (mySecs%4>1) litLED*=nleds; // also toggles light between ends (red-grn)
      refreshWait(1);
    }
    if (ESC || btn4) break;

    t0 = millis();  // start timing on Button pushed,
    // then use human originated action Entropy to directly effect the resulting answer
    litLED = 0;
    cnt = 0; maxCnt = 200;

    while (Btn) { // ----------- wait for release
      priorBtn = btnNum;
      scanBtns();
      keyCode = priorBtn;
      if (ESC || btn4) return; // time to leave
      if (ans > 0) cnt += 2; // posible responses ... lasts 1/3 as long as dark
      //if (keyCode==1) for (int i=1; i<=nleds; i++) dim[i]=1; // cause backdrop of 1/2 dim level w/halfDim=T
      if (cnt++ > maxCnt) { // produces random potenial appearing responses
        cnt = 0;  // prepare for new count with a new max
        maxCnt = (500 + 100 * random(7));
        if (keyCode>1) maxCnt=maxCnt/3;
        num = 2 + 2 * keyCode; //kc=1: 4  =2: 6  =3: 8
        ans = (ans) ? 0 : random(0, num + 1);
        if (ans == 0) clearDisp();
        else if (keyCode == 1) popColor(ans);
        else if (keyCode == 2) set_LEDs(dicePat[ans - 1], 1);
        else if (keyCode == 3) dimLED = ans;
      }
      //halfDim=(keyCode==1);
      refreshWait(1);
    }

    clearDisp();
    if (keyCode == 1) {          // select one of the 4 colors
      ans = (millis() - t0) % 4; // answer is determined by their action
      refreshWait(500 + random(600));  // make them wait alittle
      showColor(ans + 1);
    } else if (keyCode == 2) {   // select one of the 6 colors , can be used as die for for game dice
      for (int i=1; i<3; i++) { // Roll the die
        set_LEDs(dicePat[random(0, 6)], 1);
        refreshWait(100 + 200*i );
        clearDisp(); refreshWait(50);
      }
      ans = (millis() - t0) % 6; // answer is determined by their action
      set_LEDs(dicePat[random(0, 6)], 2); // lite a given pattern, similar to dice, for 1-6
    } else {                     // btn3 was used, 1 of N leds
      ans = (ans + millis() - t0) % nleds;
      steps = random(ans + 4) + (3 * nleds);
      st = 60 - steps; // step time
      for (int i = 0; i < steps; i++) { // Spin around like a roulette wheel
        ans = ans % nleds + 1;
        litLED = ans;
        refreshWait(st);
        st += i;
      }
      brightOne = litLED;
    }

    while (1) {     // await the next Request
      scanBtns();
      refreshWait(1);
      if (ESC) return;     // ESC when done
      if (Btn) break;      // go again
    }
  }
}

// ***************************  Group 4  *******************************
// ------------------- Show off your Skills ----------------------------

void  playThatTune()  { // replay a set of notes played for you.
  // starts with one then 2, 3 ...
  // each LED spot is like a key on a keyboard (Low to left, high to the right)
  // b2 < > b3     b4 to play    b1 to rehear the taget set note series

  byte ninx, kinx, n, reps, nreps, errs, maxNotes;
  int dur, offset, xfact;
  byte note[21];  // should be much more than enough

  ver = getOption(4);  // the higher version # the closer the notes are to each other
  //int ver = 1;
  if (ver == 0) return;

  // xfact=(ver==4)? 50 : 100*(4-ver);
  //xfact = 50*(5-ver) +25;
  //offset = (xfact > 150) ? 50 : (xfact < 100) ? 250 : 150;
  xfact = 50*(6-ver) +25;
  offset = (xfact > 200) ? 125 : (xfact < 150) ? 325 : 225;
  refreshWait(500);
  Serial.print("offset: ");  Serial.print(offset);  Serial.print("  xfact: ");  Serial.println(xfact);
  for (n = 1; n <= nleds; n++)  {
    litLED = n;
    ModSineWav(offset + xfact * n, 350); refreshWait(50); // run through the scale of notes (use ModSineWav over genTone for better notes)
  }

  level = 1;
  litLED = nreps = 0;
  while (1) {
    refreshWait(1000);
    clearDisp();
    maxNotes = level;
    reps = (level < 4) ? level : 3;
    errs = 0;
    for (n = 1; n <= maxNotes; n++) {
      note[n] = random(nleds) + 1;
    }
    if (note[1] == note[2]) continue; // try again

    for (n = 1; n <= maxNotes; n++)  {
      ModSineWav(offset + xfact * note[n], 500);  // play the Tune
      refreshWait(100);
    }
    ninx = 1;      // note index
    kinx = nleds / 2; // key selection inx

    while (ninx <= maxNotes) {
      scanBtnWait(1);  //  refresh disp + scanBtns + delay
      if (ESC) return;
      if (btn1Pressed) {  // replay notes to refresh their memory
        ///volHigh = !volHigh;
        // before first try or ver=1: replays are free else needed replays count as errors
        if (!(ninx == 1 || ver == 1))  errs++;
        for (n = 1; n <= maxNotes; n++) {
          dur = (n == ninx) ? 750 : 500;
          ModSineWav(offset + xfact * note[n], dur); refreshWait(100);
        }
      }
      // user to select note 1-8 with btn2 < & > btn3
      if (btn2Pressed) kinx = (kinx == 1) ?  nleds : kinx - 1; // --
      if (btn3Pressed) kinx = (kinx % nleds) + 1; // ++
      if (btn4Pressed) {  // they select the note related to the current lit Led
        ModSineWav(offset + xfact * kinx, 500);
        refreshWait(100);
        if (note[ninx] == kinx) { // they got it Right
          ninx++;
        } else {                // they got it Wrong
          beep(50);
          errs++;
        }
      }
      litLED = kinx;
      if (Btn) wait_BtnRelease();
    }

    // completed a rep at level
    playNote(3); playNote(6); // taHah audio only version
    if (errs == 0) nreps++;
    if (nreps == reps) {
      showLevel(level); // acknowledge completed level
      level++;
      nreps = 0;
    }
  }
}

//----------------  React_Now - React Quickly as LEDs light up
void React_Now() { // See how fast you a button when the first LED lights up
  // 1st LED is lit for 100 msecs, additional LEDs lite every 25ms //50ms (was 20ms for greater challendge)
  // challenge: if one pushes & holds before the first LED they get a wrong 'win'; how to fix ?
  int min_Period = 100, step=25;
  int LEDinx;

  //  btnInput = button[0];   // use the first button on the left of the four
  while (true) {
    // insure that the button is not already pressed
    while (Btn && !ESC) scanBtns();
    if (ESC) return;

    clearDisp();  // insure all LEDs are off
    scanWait(random(2000, 4000));      // wait for 2 - 5 seconds, while doing Btn scans
    lit[1] = 1; 
    //setColor(5);  // hit em with the lights
    refreshDisp();
    t0 = millis();
    //beep(15); // takes 2x msecs for 30ms
    LOUDbeep(20);
    refreshWait(min_Period - 20 - step); // account for the 30ms beep time and 'step'ms in for loop
    setColor(0);

    for (LEDinx = 1; LEDinx <= nleds; LEDinx++) {
      lit[LEDinx] = true;
      //refreshWait(step);
      //scanBtns();
      scanBtnWait(step);  // scan while waiting, for better resolution
      if (BtnPressed && !ESC) { // they pressed the button
        brightOne = LEDinx;
        Serial.print(millis()-t0);  Serial.println(" msecs");
        refreshWait(3000);
        break;
      }
    }
    if (LEDinx > nleds) beep(100);
    if (ESC) break;
  }
}

// --------------- See It ... Catch It !   (aka Capture & Release)
// Try capturing all 4 fleeting blips (later releasing them) one at a time, without fault.
// A minimum of 4 button presses is required for a ferfect score of 4.
// When done (all captured or released) your score is given (sum of hits, misses + failed oppertunities).
void SeeIt_CatchIt() {    // Capture LED lite states, to get all 4 color LEDs lit then unlit
  int time_given = 450; // within given time
  byte n;
  byte cnt = 0, lcnt;
  byte goal = 4;
  byte tally; // progress towards goal

  int ver = 1; // =1 for lenient, =2 for strickly the right & only the right btn pressed (blocking multi-presses)
  clearDisp();  // insure all LEDs are off
  while (true) {
    //refreshWait(random(1000,4000));       // wait for 1 - 4 seconds
    do {
      wait_BtnRelease();
      Serial.print(" "); Serial.println(cnt);
      standby(random(1000, 4000));  // refreshing Disp for 'msec' or btn press event
      if (Btn && lcnt == cnt) cnt++; // penalize unless they just had one
      if (ESC) return;
    } while (Btn);

    //n = random(4);    // pick an LED
    n = (random(1, 4) + n) % 4;   // pick an LED, other than the last one picked
    if (tally == 3 && (goal != 0) == lit[2 * n + 1]) n = random(0, 4); // if its already whats wanted try for another
    brightOne = 2 * n + 1;
    refreshWait(time_given);
    lcnt = cnt;
    scanBtns();   // updates btn[] emeadiately (for ver=2 should call stableBtns())
    // did they catch it ?
    if (btn[n + 1] && !(ver == 2 && (btns || !BtnChanged))) {
      cnt++; lcnt = cnt;
      lit[2 * n + 1] = !lit[2 * n + 1];
      beep(100, 100); boop(400); // ta Da!
      while (Btn) scanBtns(); // stay until released
    } else {
      if (Btn || (lit[brightOne] == (goal == 0))) cnt++; // wrong one or an oppertunity waisted
    }
    brightOne = 0;

    tally = lit[1] + lit[3] + lit[5] + lit[7];
    if (goal == 0) tally = 4 - tally;
    if (goal == (lit[1] + lit[3] + lit[5] + lit[7])) { // completed the goal
      showLevel(cnt);  // hoe many did it take
      refreshWait(500);
      if (cnt == 4) {
        brightOne = 1;  // indicate best Win!
        beep(150, 100);
        brightOne = 0;
        beep(500);
      }
      goal = (goal == 0) ? 4 : 0;
      time_given = time_given - (time_given - 200) / 5; // increase chalenge
      cnt = 0;

      refreshWait(500);
      clearDisp();
      dimFlash = 1;
      //while (!btnPressed()) refreshWait(1); // wait to start a new Go.
      do {          // wait to start a new Go.
        scanBtns(); refreshWait(1);
      } while (!(BtnChanged && Btn != 0));
      dimFlash = 0;
      if (goal == 0) {
        lit[1] = lit[3] = lit[5] = lit[7] = 1;
      }
      refreshWait(500);
    }
    scanBtns();
    if (ESC) break;
  }
}

// ---------------------------------------------------------------------
void Balance() {  // keep LED-Ball near the middle, on Simulated tilt-incline of randomly varying magnitude.
  int grade, manAdj;
  int msDelay;
  int uptrend, dntrend;
  unsigned short int pos, ball;  // short is for RP2040 
  int dspFrame, lstFrame;
  byte ledScale;
  int x;
  // old bug: when 'pos' crosses 0 it skips over the 'nled' (8th) LED
  
  ledScale = (nleds < 10) ? 40 : 30; //  make it easier ... (larger number slows action down
  loopCnt = 0;

  while (1) {
    // setup game
    clearDisp();
    wait_BtnRelease();   // wait for no button pressed

    pos = (nleds / 2) * ledScale;
    grade = 0;
    playing = true;
    while (playing) {
      // ----------------  determine input adjustment Rate
      if (loopCnt % 26 == 0 || BtnPressed) {
        manAdj = -(2 * btn1 + btn2 + (btn1 && btn2)); // for b1:-2 b2:-1 both:-3
        manAdj += (2 * btn4 + btn3 + (btn4 && btn3));
        grade +=  manAdj * 1;
      }
      if (loopCnt % 10 == 0) {
        pos += grade;
        while (pos>=65280) pos += grade; // patch to fix wrap-around problem crossing 0
        ball = (pos / ledScale);
        dspFrame = ball / nleds;
        if (lstFrame != dspFrame) Beep(30);
        lstFrame = dspFrame;
      }

      //----------------- UPDATE Display
      loopCnt++;
      clearDisp();
      ball = ball % nleds;
      lit[ball + 1] = 1;

      // set update timing
      msDelay = (nleds < 10) ? 5 : 4; // slow it down some for nleds<10
      scanBtns();
      if (Btn) refreshWait(msDelay);
      else noBtnDelay(msDelay);
      if (loopCnt % 26 == 0) { // every 100-125 mses
        //  ------------------  check tilt-incline magnitude and apply it
        // determine grade
        if (uptrend > 5) grade += 1; // increase incline
        if (dntrend > 5) grade -= 1; // decrease incline
        uptrend = dntrend = 0;
      } else {
        x = analogRead(DETECTOR); // read only every 2-5 msecs
        x += random(1000);        // use both real & psudo world
        if ((x % 5) == 1) ++uptrend;
        if ((x % 5) == 3) ++dntrend;
      }
      if (btn4 && btn1) {
        return;
      }
    }
  }
}

// =====================================================================
void Shuffleboard() { // Inline Shuffleboard. Land a ball in each posible "target" spots to the right.
  // Hold Btn1 to swing, and let go to throw.
  //Land a puck on each possible "target" spots to the right. If already occupied, it gets knocked out.
  // Hold Btn1 then let it go to launch your puck. Longer you hold further your 'puck' goes.
  int player = 0;
  int i, cnt;
  int lastCnt;
  int travel, rate, thrust;
  bool rolling, newPressed, released, btnDown;
  byte ver;
  byte row, lastS;

  //  ver=get_Option(2,1);
  ver = getOption(2);
  if (ver == 0) return;
  loopCnt = 0;
  level = 1;

  clearDisp();
  refreshWait(500);

  while (1) {
    // setup game-level
    wait_BtnRelease();   // wait for no button pressed
    cnt = 0;

    clearDisp();
    playing = true;
    while (playing) {
      dimFlash = 1;  // prompt to wined up & throw
      travel = 0;
      thrust = 0;
      rolling = false;
      turn = 1;   // # of rolled balls
      while (turn) {
        bool pressed = btnPressed();
        if (btn4 && btn1) return; // uses btn4+Btn1 to exit
        btnDown = (btn1 || btn4);
        if (btnDown && !rolling) {
          thrust++;
          spriteOne = 1;
        } else {
          spriteOne = 0;
        }

        released = (!btnDown) && BtnChanged;
        if (released) {   // the ball takes flight
          rolling = true;
          dimFlash = 0;
          //Serial.print("thrust: "); Serial.println(thrust);
          rate = (thrust + 750) / 15; //for thrust=7500 gives 100 which results in 10 spots in one second
          rate = rate / 4; // patch so we can update more often for smoother fast motion balls
        }

        if (rolling && (loopCnt % 50) == 0) { // was %200 before patch above
          if (thrust <= 30) {         // coming to a stop
            if (!dim[litLED] && !(ver == 2 && row != 2)) { // landed on a spot
              dim[litLED] = true;
              boop(40);
            } else {      // hit a laying ball
              boop(20); beep(20);
              if (ver == 1 || row == 2) dim[litLED] = false;
            }
            lastCnt = cnt;
            cnt = 0;
            for (i = 2; i <= nleds; i++) cnt += dim[i];
            if (cnt == nleds - 1) {
              winner(1);
              playing = false; // reset, due to winner()
            } else if (cnt == 0 && lastCnt) {
              lost();
              playing = false;
            }

            turn = 0;
            refreshWait(1000);
            litLED = 0;
            continue;
          }

          travel = travel + rate;
          thrust -= rate;
          litLED = travel / 100;        // (note: w/ISR use there could be a race condition)
          row = litLED/nleds + 1;
          // if(litLED!=lastS) {lastS=litLED; Serial.print(litLED); Serial.print("  "); Serial.println(row);}
          litLED  = (litLED % nleds) + 1; // keep it in proper range 
        }

        //================ UPDATE Display
        loopCnt++;
        refreshWait(1);
        // if 'esc' button held down ... Exit
        if (btn4 && btn1) {
          return;
        }
      }
    }
  }
}

void Memory() {  // There are 4 pairs of items (each a particular sound and illumination), Find the matches.
  byte mem[nleds + 2], ledType[4], sndType[4];
  byte shw[nleds + 2]; // whats showing
  byte sndOffset, ledOffset;
  int i, j, item, tries;
  byte s1, s2, loc, sum, ledState;
  bool firstSel;
  unsigned long randSeed;

  // for use with < 8 leds a ver4 could rotate the whole field 1 cell after each attempted match up.
  //                              this adds to the other wise fairly easy to become very difficulty.

  byte ver = getOption(3);  // ver 1: unique Illumination and Audio
  // ver 2: Illination Ony, No audio
  // ver 3: Audio Only, No Illumination
  if (ver == 0) return;

  randSeed = micros() / 4; // note millis() did not do the job
  randomSeed(randSeed); // make the challenge always different
  Serial.print("random Seed:  "); Serial.println( randSeed);
  mem[nleds + 1] = 0;
  glowEnable = true;
  while (!ESC) {
    clearDisp();
    wait_BtnRelease();
    firstSel = true;
    loc = 1;
    flashLed = 1;
    tries = 0;
    for (i = 0; i <= nleds + 1; i++) {
      mem[i] = shw[i] = glow[i] = 0;
    }
    item = 0;  // valid items 1-4

    // for (i = 1; i < nleds; i++) { // ------------  place items into slots  [done utilizing random()]
    //   while (mem[i] != 0) i++;
    //   if (i > nleds) break;
    //   item++;
    //   mem[i] = item;
    //   do {  // find a spot for its match
    //     j = random(i + 1, nleds + 1);
    //     if ((i == 1 || i == 3) && j == i + 1) j = random(i + 1, nleds + 1); // reduce chance of matches 1&2 and 3&4
    //   } while (mem[j]);
    //   mem[j] = item;
    // }
    for (i = 0; i < nleds; i++) { // ---------------  place items into slots  [now done utilizing PRIG()]
      mem[PRIG(i++,gameID,nleds)+1] = ++item;    // here there is no need to check if mem[index] is used, & it hadn't needed to be pre cleared to 0
      mem[PRIG( i ,gameID,nleds)+1] = item;
    }

    ledOffset = random(4);
    sndOffset = random(4);

    for (i = 1; i <= nleds; i++) {
      Serial.print(i);
      Serial.print("  sound:  "); Serial.print(mem[i]);
      Serial.print("  light:  "); Serial.println(mem[i]);
    }
    Serial.println();

    // --------- player(s) try to match up the items, testing thier Memories
    while (!ESC) {
      if (btn1 && btn4) return; // fast exit

      if (Btn) {
        if (btn2Pressed) loc = (loc == 1) ?  nleds : loc - 1; // -- move left Allow 0:debugging
        if (btn3Pressed) loc = (loc % nleds) + 1; // ++ moves right
        if (BtnPressed) flashLed = loc;
        if (btn1Pressed) {    // wait for slot selection
          if (shw[loc]) {
            buZZ(15); // already choosen
          } else {
            // its the First slot selection
            if (firstSel) {
              firstSel = false;
              tries++;
              s1 = loc;
              shw[loc] = mem[loc];
              flashLed = 0;
              if (ver != 3) { // ---------  show the selected item
                switch ((mem[loc] - 1 + ledOffset) % 4)
                {
                  case 0: dim[s1] = 1;
                    break;
                  case 1: lit[s1] = 1;
                    break;
                  case 2: brt[s1] = 1;
                    break;
                  case 3: glow[s1] = 1;
                }
              } else {
                brt[s1] = 1;
              }
              if (ver != 2) {
                switch ((mem[loc] - 1 + sndOffset) % 4)
                {
                  case 0: boop(75);
                    break;
                  case 1: beep(75);
                    break;
                  case 2: Beep(90);
                    break;
                  case 3: buZZ(60);
                }
              }
              refreshWait(250);
            }

            // its the Second slot selection
            else {
              firstSel = true;
              s2 = loc;
              shw[loc] = mem[loc];
              flashLed = 0;
              if (ver != 3) { // ---------  show the selected item
                switch ((mem[loc] - 1 + ledOffset) % 4)
                {
                  case 0: dim[s2] = 1;
                    break;
                  case 1: lit[s2] = 1;
                    break;
                  case 2: brt[s2] = 1;
                    break;
                  case 3: glow[s2] = 1;
                }
              } else {
                brt[s2] = 1;
              }
              if (ver != 2) {
                switch ((mem[loc] - 1 + sndOffset) % 4)
                {
                  case 0: boop(75);
                    break;
                  case 1: beep(75);
                    break;
                  case 2: Beep(90);
                    break;
                  case 3: buZZ(60);
                }
              }
              refreshWait(250);

              // Do we have a match ?
              if (mem[s1] == mem[s2]) {
                refreshWait(250);
                taDah();  // YES - give very good ack ... keep them lit
                if (ver == 3) {
                  dim[s1] = 1;
                  dim[s2] = 1;
                }
                refreshWait(50);

                // Do we have a winner ?
                sum = 0;
                for (i = 1; i <= nleds; i++) {
                  if (shw[i]) sum++;
                }
                if (sum == nleds) {
                  taDah();
                  refreshWait(250);
                  showLevel(tries);
                  Btn = 0;
                  while (!Btn) scanBtnWait(1);  // wait for next button press
                  break;  // start over
                }

              } else {
                refreshWait(500);
                buZZ(15); // NO - give raspberry ... un lite them
                refreshWait(1500);   // let them ponder to remember
                shw[s1] = 0;
                shw[s2] = 0;
                flashLed = loc;
              }
              if (ver == 3) {
                brt[s1] = 0;
                brt[s2] = 0;
              }
            }
          }

          // update whats displayed
          if (ver != 3) {
            for (i = 1; i <= nleds; i++) {
              dim[i] = lit[i] = brt[i] = glow[i] = 0;
              if (shw[i]) {
                ledState = ((shw[i] - 1 + ledOffset) % 4);
                switch (ledState)
                {
                  case 0: dim[i] = 1;
                    break;
                  case 1: lit[i] = 1;
                    break;
                  case 2: brt[i] = 1;
                    break;
                  case 3: glow[i] = 1;
                    break;
                }
              }
            }
          }
        }
      }

      refreshWait(1);
      scanBtns();
    }
  }
}

void CodeBreaker() { // There's a 4 digit entry Btn sequence code. Find it, with Master-Mind like feedback.
  byte code[5];
  int i;
  byte key, good, sum;
  int tries;
  byte keys[5], entry[5], used[5];
  int waiting;

  // with <8 Leds, the ongoing status reporting will need rethinking

  while (1) {
    clearDisp();
    for (i = 4; i >= 0; i--) { // start 'flash screen'
      setColor(i);
      refreshWait(200);
    }
    keys[1] = keys[2] = keys[3] = keys[4] = 0;
    for (int i = 1; i <= 4; i++) {
      code[i] = random(4) + 1;
      keys[code[i]]++;  // keep cnt of # of each keys (btns) used in the code
    }
    Serial.print(" Code:  ");
    for (int i = 1; i <= 4; i++) {
      Serial.print(code[i]);
      Serial.print("  ");
    }
    Serial.println();

    tries = 0;
    while (!ESC) {
      used[1] = used[2] = used[3] = used[4] = 0;
      entry[1] = entry[2] = entry[3] = entry[4] = 0;
      tries++;;
      good = 0;
      // -------- get a 4 key attempt
      for (key = 1; key <= 4; key++) {
        waiting = 0;
        while (!Btn) {    // wait for next button press
          scanBtnWait(1);
          if (++waiting > 10000) { // if nothing for 10 secs, get their attention
            for (i = 4; i >= 0; i--) {
              setColor(i);
              refreshWait(100);
            }
            waiting = 0;
          }
        }
        entry[key] = btnNum;
        setColor(btnNum);
        if (btn1Pressed) phoneTone(1);
        if (btn2Pressed) phoneTone(4);
        if (btn3Pressed) phoneTone(7);
        if (btn4Pressed) phoneTone(10); // would be the '0' on the dial
        if (code[key] == entry[key]) good++;
        used[btnNum % 5]++;
        setColor(0);
        wait_BtnRelease();
        if (ESC) return;
      }
      Serial.print(" Good: "); Serial.print(good);

      // -------- Check if it is right
      if (good == 4) {
        winner(1);
        showLevel(tries);
        refreshWait(2000);
        break;
      }

      // -------- report how well they did
      sum = 0;
      for (key = 1; key <= 4; key++) {
        if (keys[key] > used[key]) sum += used[key];
        else sum += keys[key];
      }
      Serial.print("  keys used: "); Serial.println(sum);
      delay(150);
      if ((sum + good) == 0) buZZ(200);
      else {
        showLevel(good);
        for (i = nleds; sum > 0; i--) { // to the right show # of correctly selected keys
          lit[i] = true;
          sum--;
        }
        refreshWait(3000);
        clearDisp();
      }
      //refreshWait(1);
      refreshWait(1);
      scanBtns();

      wait_BtnRelease();
      if (ESC) return;
    }
  }
}

void Pairs_Puzzle() { // Flip a combination of pair sets (driven by Btns) to null pattern on display
  // modified to prevent lucky solution in 4 or less.  Can be solved in as little as 3.
  byte pairs[5];  // containd bit pattern of two sets of colors of LEDs, associated to btn1-4
  byte sets[4] = {0x03,0x0C,0x30,0xC0};
  byte u[5], bs[5];
  byte s, ss, i;
  byte LEDisp, last, t, tmp;
  int attemps;
  bool conflix;
  
  while (!btn4) {
    clearDisp();
    refreshWait(500);
    attemps=0;

    // setup new game
    do {
      gameID++;
      for (i=0; i<5; i++ ) {pairs[i]=0; u[i]=0; bs[i]=0;}
      conflix=false;
      last=0;
      // assign a pair of colors to each of the 4 buttons
      for (int i=1; i<=4; i++) {
        s=PRIG(i-1,gameID,4);  // pick a set
        pairs[i] |= sets[s];
        for (int ii=1; ii<=4; ii++) {  // pick Second different set & not already used as a second
          ss=PRIG(ii-1,gameID/2,4);
          if (ss!=s &&  u[ss]==0 && bs[ss]!=s+1) {
            pairs[i] |= sets[ss];
            u[ss]=1;
            bs[s]=ss+1;
            //Serial.print(i); Serial.print("    ");  Serial.println(pairs[i], BIN);
            break;
          }
          if (ii==4) conflix=true;    // didn't manage a valid collection of 4
        }
      }
    } while (conflix);

    // pick a starting pattern
    LEDisp=0;
    s=PRIG(i++,gameID+1,4);  // pick a set
    LEDisp |= sets[s];
    s=PRIG(i++,gameID+1,4);  // pick a seccond different set
    LEDisp |= sets[s];
    //Serial.print("st:  ");  Serial.println(LEDisp, BIN);
    // tried where it has no = button set pattern, but that was more predicted in solving, so boo
    //defaunt: Serial.print("Number of searches for starting pattern:  "); Serial.println(i/2);
    set_LEDs(LEDisp,2); // lite the starting LED patetern

    // -----------  GO Time. player strives to clear the screen (LEDs) by flipping pairs of colored LEDs
    while (!ESC) { 
      // process their attemps
      if (BtnPressed) {
        attemps++;
        if (attemps<3 && LEDisp==pairs[btnNum]) {  // ------ disallow lucky early solulions
          t = ((btnNum-1) + PRIG(0,gameID+41,3)+1) % 4 +1;  // trade the pattern for this button with another
          if (t==last) t = ((btnNum-1) + PRIG(1,gameID+41,3)+1) % 4 +1;
          tmp=pairs[btnNum]; pairs[btnNum]=pairs[t]; pairs[t]=tmp;   // swap
          //boop(150);  for testing only
          //Serial.print(btnNum); Serial.print("    ");  Serial.println(pairs[btnNum], BIN);  // show your work
          //Serial.print(t); Serial.print("    ");  Serial.println(pairs[t], BIN);
        }
        last = btnNum;
        LEDisp ^= pairs[btnNum];
        clearDisp();
        set_LEDs(LEDisp,2);
      }
      if (LEDisp==0) {     // check to see if they did it
        taDah(); 
        showLevel(attemps);  refreshWait(1500);
        stableBtns(); // clear out old button state
        break;  // game completed ... go for another
      }
      refreshWait(1);
      scanBtns();
    }
  }
}

// ***************************  Group 5  *******************************
// ----------- Demos, Challenges, Activities and Applets ------------------
void  Support_Demo() { // Demonstrates use of Hardware support functions, plus Leds and audio level Scales.
  // delays & Display Refresh timing is critical to proper display, as a changing myMillis is used to fill the display
  // when using interupt driven display updates this is not an issue.
  byte lvl, c, maxLvl, LedL;
  int i;
  bool waiting;
  byte n, lastN, lastC, nLit;
  unsigned int tCnt;
  unsigned int ledTime;
  bool WelcomeScreen = true;

  //  demostrate the usefulness of set_LEDs()
  for (i = 0; i < 8; i++) {
    refreshWait(100);
    set_LEDs(1 << i, 1);
  }
  for (i = 0; i < 8; i++) {
    refreshWait(100);
    set_LEDs(1 << i, 2);
  }
  for (i = 0; i < 8; i++) {
    refreshWait(100);
    set_LEDs(1 << i, 3);
  }
  for (i = 0; i < 8; i++) {
    refreshWait(100);
    set_LEDs(1 << i, 0);
  }
  refreshWait(500);

  //    escCnt=ESC=0;
  ESC = 0;
  while (!ESC) {

    clearDisp();
    lvl = 0;
    wait_BtnRelease();
    waiting = true;
    WelcomeScreen = true;
    //------------------------ Phase 1 --------------------------------------------
    if (Verbose) Serial.println(" Demonstrates the (Mini_STEM_Platform) Standard Game sounds with lights ");
    if (Verbose) Serial.println("   Btn1: Boop-Dim,  Btn2: Beep-Lit, Btn3: BuZZ-Bright  ");
    while (!btn4) {
      scanBtns();

      if (WelcomeScreen && !BtnPressed) {  // display splash display until a new button presses
        for (i = 1; i <= nleds; i++) {
          dim[i] = true;  // set all LEDs dim
          lit[i] = ((msCnt / 333 + i) % 2); // flash odd & even LEDs in opposition
        }
        //      refreshWait(1);
      } else WelcomeScreen = false;

      if (BtnPressed) {
        clearDisp();
        for (i = 1; i <= nleds; i++) { // set all LEDs to a given level
          if (btn1Pressed) dim[i] = true;
          if (btn2Pressed) lit[i] = true;
          if (btn3Pressed) brt[i] = true;
          if (btn4Pressed) clearDisp();
        }
      }

      // produce sound while btn1-4 is held
      while (Btn) {
        if (btn1) boop(10);
        if (btn2) bop(10);
        if (btn3) beep(10);
        if (btn4) {
          buZZ(700);
          break;
        }
        scanBtns();
      }

      refreshDisp();
      delay(1);
      if (ESC) break;
    }

    wait_BtnRelease();
    //------------------------ Phase 2 --------------------------------------------
    if (Verbose) Serial.println(" Step thru Expanded Game sounds with lights. Based on additions utils in this sketch. ");
    while (!btn4) {
      refreshWait(1);
      scanBtns();
      if (ESC || (btn4 && btn1)) return;
      if (waiting && !Btn) {
        // ------------- create a 'Waiting' display
        n = (myMillis / 1000) % nleds;
        if (n != lastN) { // brighten up the privious dim LED, for 2x action
          litLED = dimLED;
          lastN = n;
        }
        //c = (n&1)? 1 : nleds;                 // toogle between 1st and last Led
        //c = (n/(nleds/2)) + 2*(n%4);            // Step thru odd then even Leds
        c = (nleds / 2) * (n & 1) + (n >> 1);    // pop back&fore lower-upper section
        //c = (n&0xFC) + ((n<<1)&2) + ((n>>1)&1);  // reverse the lower two bits, ~ skippy travel
        //dimLED=c+1;
        dimLED = c + 1;
      } else {
        dimLED = 0;
        waiting = false;
        // lvls: 0=off,  1=dim,  2=lit,  3=brt,  4=running flash
        if (btn2Pressed) lvl = (lvl == 0) ?  4 : lvl - 1; // -- down
        if (btn3Pressed) lvl = (lvl + 1) % 5;         // ++ up
        if (BtnPressed && lvl == 0) clearDisp();
        dim[myMillis % nleds + 1] = (lvl == 1); // note this is dependant on Disp Updates, to fill over time
        lit[myMillis % nleds + 1] = (lvl == 2);
        brt[myMillis % nleds + 1] = (lvl == 3);
        if (lvl == 4) {
          for (i = 1; i <= nleds; i++) { // -------------  flash odd & even LEDs in opposition
            brt[i] = (((myMillis / 200) % 2) == (i & 1));
          }
        }

        if (Btn && !btn4) {
          if (lvl == 1) genTone(150, 28);
          if (lvl == 2) boOop(30); // 250Hz boop tone with Vibrato
          //if (lvl==3) beep(20);
          if (lvl == 3) LOUDbeep(19); // extra Loud beep
          if (lvl == 4) Beep(20); // mixed freq. beep
          //if (lvl==0) buZZ(41);
          if (lvl == 0) {
            boop(4);  // not as harsh as 'buZZ'
            beep(1);
            bop(2);
          }
        }
        //            refreshWait(1);
      }
    }

    wait_BtnRelease();
    //------------------------ Phase 3 ------------------------------------------------
    if (Verbose) Serial.println(" Step thru a gradient scale of 24 tones and 16 levels of lit ");
    clearDisp();
    lvl = 0;
    maxLvl = 24;
    if (!ESC) wait_BtnRelease();
    if (ESC) return;
    waiting = true;
    while (!btn4) {
      refreshWait(1);
      scanBtns();
      if (waiting && !Btn) {
        if (myMillis % 50 == 1) { // upate every ~50 ms
          nLit = (nLit + 1) % 32;
          if (nLit == 0) nLit++;
          levelOn[0] = (nLit <= 16) ? nLit : (16 - (nLit - 16)); // goes 1-16-1 & repeats
          for (i = nleds; i > 0; i--) { // Fade-Wave for wait banner
            levelOn[i] = levelOn[i - 1]; // scroll light levels to the right
          }
        }
      } else {
        if (waiting) {
          clearDisp();   // clears at the hardware level too, now
          //refreshDisp();
          waiting = false;
        }
        if (btn2Pressed) lvl = (lvl == 0) ?  maxLvl : lvl - 1; // -- down
        if (btn3Pressed) lvl = lvl % maxLvl + 1; // ++ up  (lvl is 1-maxLvl)
        //            levelOn[myMillis%nleds+1]=lvl;
        levelOn[ledTime % nleds + 1] = (16 * lvl + 8) / maxLvl; // spread accross 16 levels,

        LedL = (lvl - 1) % nleds + 1;
        if (btn2Pressed || btn3Pressed) {
          blinkLed =  (LedL > 0) ? (LedL - 1) % nleds + 1 : 0;
        }

        if (BtnPressed && !btn4)  playNote(lvl, 200);
        //while (btn1) {playNote(lvl,200); scanBtns();}  // 100 for less notable breakup
        // this faster test reduces tone breakup (but is not in the spirit of HW support include)
        while (digitalRead(button[0]) == PRESSED_STATE) {
          playNote(lvl, 200);
        }

        ledTime++;
      }
    }

    wait_BtnRelease();
    //------------------------ Phase 4 ... TBD ------------------------------------
    //while (!btn4) {
    //}
    
    if (!ESC) wait_BtnRelease();
  }
}

void Illumination_Challenge() { // DIY LED Illumination Pattern setup (plus a challenge)
  // Challenge: recreation of Led illumination pattern as flawlessly as possible
  // Holding Btn1 will test for & display target pattern of lights. 
  // Recreate the pattern (with B2<>B3 B4:chance) with minimum Btn1s to win.
  int n;
  bool equal;
  long pattern;
  byte patcell[14];
  int divr, state;
  bool trying, init;
  byte lastBtn;

  glowEnable = true;
  while (1) {
    init = true;
    cursorPtr = 1;
    lastBtn = 2; // so cursor shows
    clearDisp();
    pattern = random(10001, 2147483647L);
    Serial.print("Pattern Number: "); Serial.println(pattern);
    divr = 2;
    for (n = 0; n <= nleds; n++) {
      cell[n] = 0;
      divr += n;
      patcell[n] = (pattern / divr) % 6; // all psuedo random setting are dirived from a single pattern number
      if (patcell[n] > 3 ) patcell[n] = (pattern / (divr + 1)) % 6; // reduces freq of arc & glow
    }

    trying = true;
    while (trying) {
      refreshWait(1);
      //scanBtns();
      // ------------------- button reporting, Debuging
      if (btnPressed()) {
        lastBtn = btnNum;
        if (DEBUG) {
          if (btn2Pressed) {
            for (int i = 1; i <= nleds; i++) {
              Serial.print(i); Serial.print("  pat: ");  Serial.print(patcell[i]);
              Serial.print("  cell: ");  Serial.println(cell[i]);
            }
          }
        }

        if (btn1Pressed) {  // --------- View/Test the desired Illumination Pattern
          equal  = true;
          for (n = 1; n <= nleds; n++) if (cell[n] != patcell[n]) equal = false;
          if (equal) {
            taDah();
            refreshWait(1000);
            while (btn1) {
              scanBtns();
              refreshWait(1);
            }
            trying = false;
          } else {
            if (!init) boop(25);
          }
        }
        // ------------------ LED change support
        if (btn2Pressed) cursorPtr = (cursorPtr == 0) ?  nleds : cursorPtr - 1; // --  0-nleds
        if (btn3Pressed) cursorPtr = ((cursorPtr + 1) % (nleds + 1)); // ++
        if (btn4Pressed) cell[cursorPtr] = (cell[cursorPtr] + 1) % 6;
        if (btn4) init = false;
      }

      // ---------------  update the display
      //clearDisp();
      for (n = 1; n <= nleds; n++) {
        state = (btn1) ? patcell[n] : cell[n];
        dim[n] = (state == 1);
        lit[n] = (state == 2);
        brt[n] = (state == 3);
        arc[n] = (state == 4); // added arc & glow Sept 2021
        glow[n] = (state == 5);
      }
      blinkLed = (lastBtn == 2 || lastBtn == 3) ? cursorPtr : 0;

      if (ESC) return;
    }
  }
}

void Automated_Art() { // would-be 'LIFE' simulation in 1 dimension
  // =============================  Automated Abstract Art
  long pattern;
  int dir[4];
  int spritePtr;    // sprite that is not seen
  int deltaOne, deltaPtr;
  unsigned int beet;
  bool gMute;
  byte speed;
  int spdCnt[] = {1000, 500, 250, 100};
  bool pause;

  deltaPtr = 0;
  loopCnt = beet = 0;
  gMute = false;
  speed = 1;

  while (1) {
    loopCnt++;
    refreshWait(1);
    scanBtns();
    if (ESC) break;

    if (btn1Pressed || deltaPtr == 0) {
      pattern = random(10001, 2147483647L);
      randomSeed(pattern);
      spriteOne = random(nleds) + 1;
      spritePtr = random(nleds) + 1;
      deltaOne = random(1, nleds);
      deltaPtr = random(1, nleds);
      Serial.print("Pattern Number: "); Serial.print(pattern); Serial.print(" +D: "); Serial.print(deltaOne); Serial.print(" -D: -"); Serial.println(deltaPtr);
    }
    if (btn2Pressed) {  // toggle genetic Mutation
      gMute = !gMute;
      if (gMute) beep(50);
    }
    if (btn3Pressed) {  // speed shift
      speed = (speed + 1) % 4;
    }
    while (btn4Pressed || pause) {  // pause func
      if (btn4Pressed) pause = !pause;  //toogle in&out of pause
      scanBtns(); refreshWait(1);
      if (ESC) return;
    }

    for (int i = 0; i < 4; i++) dir[i] = 0;
    if (loopCnt % spdCnt[speed] == 0) {      // consider influence over time
      Serial.print(" ON: "), Serial.print(spriteOne);  Serial.print(" OFF: "), Serial.println(spritePtr);
      lit[spriteOne] = true; // leave a 'on' trail
      lit[spritePtr] = false; // turn em off
      spriteOne += deltaOne; // ++ go right
      spritePtr -= deltaPtr; // -- go left
      if (spriteOne > nleds)  spriteOne = spriteOne - (nleds + 1);
      if (spritePtr < 0)      spritePtr = spritePtr + (nleds + 1);

      //was         if (gMute && (++beet)>(nleds*nleds) && spritePtr==0) {  // time to alow a mutation
      if (gMute && (++beet) > (4 * nleds)) { // time to alow a mutation
        if (random(2))  deltaOne = random(1, nleds);
        else            deltaPtr = random(1, nleds);
        beet = 0;
      } else {      // wrap-around spriteOne now, spritePtr will after one more beet, so that we see upto a nleds^2 sequence
        if (spriteOne == 0) spriteOne = deltaOne;
      }
    }
  }
}

void  tuneLightOrgan()  {  // Tool to form and evaluate future note sequences and game sounds
  // option 1: choose 15 piano notes G3-G5,   number of notes is up to the count of LEDs
  //        2: select Frequencies from a range of 47  (131 - 13,190 Hz)
  byte note, notes[nleds + 2]; // should be much more than enough
  byte lvl, ninx, n, maxNote, numN, n0 = 4;
  byte playCnt;
  int freq, dur;

  ver = getOption(2);
  if (ver == 0) return;

  maxNote = (ver == 1) ? 15 : 47; // number of notes in the scale used
  refreshWait(500);
  freq = 130;
  for (n = 0; n < 15; n++)  { // give a sample of notes
    litLED = n % nleds + 1;
    notes[litLED] = 0;
    if (ver == 1) {
      playNote(n0 + n + 1, 200);  // run through the scale of notes
      refreshWait(50);
    }
    else genTone(freq, 150);
    freq = 14L * freq / 10;
  }

  refreshWait(1000);
  clearDisp();
  ninx = 1;      // note index
  lvl = 0;
  while (1) {
    if (ESC || (btn4 && btn1)) return;
    if (btn1Pressed) {  // ----------------------  play notes at desired tempo
      t0 = millis(),
      wait_BtnRelease();
      dur = millis() - t0;
      Serial.print("Duration of each note: "); Serial.println(dur);
      //        volHigh = !volHigh;
      clearDisp();
      numN = nleds;
      while (!notes[numN]) numN--;  // prepare to play only the provided notes
      for (n = 1; n <= numN; n++) {
        levelOn[n] = (16 * notes[n] + maxNote - 1) / maxNote;
        if (ver == 1) {
          note = (notes[n] > 0) ? n0 + notes[n] : 0;
          playNote(note, dur);
          refreshWait(dur / 4);
        } else {
          note = notes[n];
          if (note < 25) freq = basicKeyNotes[note];
          else freq = basicKeyNotes[note - 23] * 10;
          genTone(freq, dur);
        }
      }
    }

    // ---------------------------  user to change note/light level 0-N with btn2 < & > btn3
    if (btn2Pressed) lvl = (lvl == 0) ?  maxNote : lvl - 1; // -- freq.+illumination
    if (btn3Pressed) lvl = (lvl + 1) % (maxNote + 1);       // ++ freq.+illumination
    notes[ninx] = lvl;
    levelOn[ninx] = (16 * lvl + maxNote - 1) / maxNote;
    while (btn2 || btn3) {
      if (ver == 1 && lvl != 0) playNote(n0 + lvl, 120);
      if (ver == 2) {
        if (lvl < 25) freq = basicKeyNotes[lvl];
        else freq = basicKeyNotes[lvl - 23] * 10;
        genTone(freq, 120 + lvl); // give high freqs added dur
      }
      //btnsNow();   // get a full state update (despite debounce)
      stableBtns();
      // with ver 2 auto step thru freqs
      if (ver == 2 && btn2) lvl = (lvl == 0) ?  maxNote : lvl - 1;
      if (ver == 2 && btn3) lvl = (lvl + 1) % (maxNote + 1);

      if (btn2 && btn3) {  // reset to null note
        lvl = 0;
        wait_BtnRelease();
      }
      if (!Btn) {
        Serial.print("Note: ");  // B2-3 released
        Serial.println(lvl);
      }
    }

    if (btn4Pressed) {  //-------------  move point of interest 1-nleds
      ninx++;
      if (ninx > nleds) ninx = 1;
      lvl = notes[ninx];
    }

    // update our custom cursor
    spriteOne = dimIt = litLED = dimFlash = 0;
    if (myMillis % 1800 < 300 || btn4) {
      if (notes[ninx] <= (2 * maxNote / 3) ) spriteOne = ninx;
      else                dimIt = ninx;
    }

    refreshWait(1);
    scanBtns();
  }
}

// ------------ Sound Effects has been moved to their own .h file --------------------- //

void  Timer()  { //  Timer ... flash & ticks secs and trumpets minutes (+ Stop-Watch functions)
  int  pos, minCnt, secs, n;
  bool toggle, timing, paused;
  unsigned int mCnt;
  unsigned long mics;
  int trimAdj;
  int alMin=0;  // the alarm minute
  bool ISR=false;  // with ISR=true uses a 1 millisecond ISR interurpt routine
                  // otherwise  uses delay() and is adjusted for @(mCnt%500 == 0) below
  //w/ISR  unsigned long mStart;

 #ifdef RP2040
  trimAdj = 13; // 16.6 shortens interval by 1/60th (making 1sec difference in a minute)
 #else
  trimAdj = 168; // imperical rate adjustment, re given device, for timing accuracy sake (MCU,overhead,V,temp. ?dependant) ?uptodate?
 #endif


  ver = getOption(4);
  if (ver==0) return;
  alMin=ver%4;  // ver 1-3 is for 1-3 min alarms, 4 results in continuous running

  while (!btn4) {
    clearDisp();
    //      xdimEnabled = true;
    //w/ISR      mStart=myMillis;
    mCnt = 0;
    minCnt = 0;
    pos = 1;
    toggle = false;
    paused = false;
    timing = true;
    while (timing) {
      secsIdle = 0; // thwart idle timeout
      if (ISR) {
        //w/ISR          while (mCnt == (myMillis-mStart)) {}  // wait for ISR to change 'myMillis'
        //w/ISR          mCnt=myMillis-mStart;
      } else {  // not ISR 1 ms wait while doing screen refresh
        //delay(1);         // WB more accurate (~61 sec minute) but lose dimming and second's ticks
        refreshWait(1);    // slowser timing (~64 sec minute)
        // but is adjusted for below @(mCnt%500 == 0)
        mCnt++;
      }

      scanBtns();
      if (ESC) return;
      if (btn1Pressed) {
        timing = false;   // Restart
        break;
      }

      if (btn4Pressed) paused = !paused;  // toggle in/out of a paused state
      if (paused) {
        wait_BtnRelease();
        while (paused) {
          // display time so far by way of a combination of bright and flashing lites
          secs = mCnt % 10000 / 1000; // seconds into this 10 sec period
          if (0 == secs) {
            spriteOne = pos;
            noBtnDelay(1000);
          } else {
            for (byte s = 1; s <= secs; s++) {
              spriteOne = pos;  noBtnDelay(50);
              spriteOne = 0;  noBtnDelay(200);
            }
          }
          spriteOne = 0;
          noBtnDelay(1000);   // wait upto  1 sec, given no buttons pressed
          //            if (btnState>0) {
          if (btn1Pressed || ESC) {
            timing = false;   // Restart or Exit
            break;
          }
          if (Btn) {
            paused = false;
            wait_BtnRelease();
          }
        }
      }
      if (ESC) return;

      if (mCnt % 500 == 0) { // once a second lite then noLite then adv the sprite
        toggle = !toggle;
        if (toggle) {
          spriteOne = 0;
        } else {
          //            pos = pos%nleds + 1;
          spriteOne = pos;
        }
      }

      //  -------------  Adjust for your MCU and setup      !!!!!!!!!!!!!!!!!!!!!!!!!
      // imperical rate adjustment,  mCnt+=16.6 shortens interval by 1/60th (making 1sec difference in a minute)
      if (!ISR && mCnt % 1000 == 500)  mCnt += trimAdj;

      if (mCnt % 10000 == 0) {  // little beep every 10 seconds
        spriteOne = pos;  // be sure it is lit
        //          if (mCnt!=60000) {
        beep(20);
        mCnt += 20;     // adj for time spent
        //          }
        pos = pos % 6 + 1; // 6*10 for 60secs / Min.
        //na          idleCnt=0;  // keep from timing out
      }
      if (mCnt >= 60000) { // every Minute
        //w/ISR          mStart=myMillis;
        clearDisp();
        mCnt = 100;
        minCnt++;   // inc Minute count
        Loud = true;
        if (alMin==minCnt) {  // ------ alarm set time hit !
          fail(); // used for alarm anouncement
          Loud = false;
          spriteOne=alMin;
          while(!btnPressed()) refreshWait(1);
          timing = false;   // Restart or Exit
          break;
        }
        for (n = 1; n <= minCnt/10; n++) {  // buZZ off the 10s of minutes
          spriteOne = nleds;
          buZZ(75); 
          spriteOne = 0; refreshWait(125);
          mCnt += 200;  // account for time spent
        }
        if (minCnt>=10) { refreshWait(200); mCnt += 200; }
        for (n = 1; n <= minCnt%10; n++) {  // beep off the minute count
          dim[n] = true;
          spriteOne = (n - 1) % nleds + 1;
          beep(75); refreshWait(125);
          mCnt += 200;  // account for time spent
        }
        Loud = false;
        litLED = spriteOne; // add extra brightness on the Led of the current Minute
        pos = 1 + mCnt / 10000;
      }
    }
  }
}

// ================
void Metronome() {  // Emulates a metronome. 20-240 BPM, Mode selection: 1/4 - 4/4 time.
  // specs: AVR: ~ Beat time Deviation: 4ms, BPM +/-2ms     RP2040: beat Devi: 1ms,  BPM +/-1ms
  int bpm = 100; // Range goes 20-240, Starts at 100bpm, good for "row row ..." or CPR
  byte count, beats = 0, bcnt;
  int ams;
  int st, pos = 1, dir = 1;
  unsigned long stime = 0;
  bool bpmChg = true;
  int hiv = 0, lowv = 99; // high & low values

  count = get_Mode(4); // select 1/4 2/4 3/4 or 4/4 time

  while (!ESC) {

    if (BtnPressed || bpmChg) {
      if (btn1Pressed) {
        bpm = (bpm % 10 == 0) ? bpm - 10 : 10 * (bpm / 10); // dec -10
        if (bpm < 20) bpm = 240;
      }
      if (btn2Pressed) bpm = bpm - 1;
      if (btn3Pressed) bpm = bpm + 1;
      if (btn4Pressed) {
        bpm = (bpm % 20 == 0) ? bpm + 20 : 20 * (bpm / 20 + 1); // inc +20
        if (bpm > 240) bpm = 20;
      }

      st = (60000 / bpm) / nleds - 1; // coarse LED step time, as there are overhead delays: -1 & fine adj later on the fly
      stime = millis();
      bcnt = 0; t0 = millis(); // reset test timing, as bpm has chgd
      pos = 1, dir = 1;
      hiv = 0; lowv = 99;
      if (bpm == (20 * int(bpm / 20)) && bpm >= 40 && bpm <= 180) {
        spriteTwo = (bpm - 20) / 20;   // when at multiple of 20 HighLight it
      } else spriteTwo = 0;

      Serial.print("bpm="); Serial.print(bpm);
      Serial.print("  StepTime="); Serial.println(st);
    }
    bpmChg = false;
    if ((msCnt - lastChg) > 1500)  spriteTwo=0;

    if ((millis() - stime) >= st) { // time to move needle
      stime = millis();
      pos = (pos + dir) % (nleds + 2);

      if (pos == 0 || pos > nleds) { // ------- end of each beat period
        ++bcnt;
        ams = (bcnt * 60000L / bpm) - (millis() - t0); // fine adj. as needed sb: 0-9 due to process rounding
        if (ams >= 0) delay(ams); // note: beat-to-beat time remains the same within ~1msec, at least with RP2040
        stime = millis(); // reset step timer
        ///// diag/TESTing ...
        if (ams > hiv && bcnt > 0) hiv = ams;
        if (ams < lowv && bcnt > 0) lowv = ams;
        /////

        if (bcnt >= bpm) { // ---- have done #BPM for a minute

          ////////// for diag/calc timing testing ...
          Serial.print(bpm); Serial.print("bpm,  deviation(ms):"); Serial.print((hiv - lowv) / 2.0); Serial.print(",   millisecs #BPM used as a 'minute': ");
          Serial.println(millis() - t0); // sb: 60,000 will be +/-1
          // auto -- for testing
          //if (bpm>20) bpm--;
          //bpmChg=true;
          hiv = 0; lowv = 99;
          //////////

          bcnt = 0;
          t0 = millis();
        }
      }

      if (pos > nleds) {
        pos = nleds;
        dir = -1;
      } else if (pos == 0) {
        pos = 1;
        dir = 1;
      }
      dimLED = pos;
      if ((pos == nleds / 2) || (pos == (nleds / 2 + 1))) brightOne = dimLED;
      else brightOne = 0;
      // crossing the middle tick ... "beat"
      if ((dir == 1 && pos == (nleds / 2 + 1)) || (dir == -1 && pos == (nleds / 2))) {
        beats++;
        if (beats == count) {
          bop(25);  // audio time must be <29
          beats = 0;
        }     // try to make a 'tick' sound
        else beepTone(2000,8);
      }
    }

    refreshWait(1);
    scanBtns();
    secsIdle = 0; // thwart idle timer
  }
}

// ------------ 
long readVcc() { // used by readVoltages()

 #if defined(RP2040)
  return (0); // does not support or unaware of how
 #else
  // Read 1.1V reference against AVcc
  // set the reference to Vcc and the measurement to the internal 1.1V reference
  //choose the right register settings for the processor in use
 #if defined(__AVR_ATmega32U4__) || defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)
  ADMUX = _BV(REFS0) | _BV(MUX4) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
 #elif defined (__AVR_ATtiny24__) || defined(__AVR_ATtiny44__) || defined(__AVR_ATtiny84__)
  ADMUX = _BV(MUX5) | _BV(MUX0);
 #elif defined (__AVR_ATtiny25__) || defined(__AVR_ATtiny45__) || defined(__AVR_ATtiny85__)
  ADMUX = _BV(MUX3) | _BV(MUX2);
 #else
  ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);  // this one for Nano, ATmega-328
 #endif

  delay(2); // Wait for Vref to settle typical: +/-.03 (1) is good @ +/-.06
  ADCSRA |= _BV(ADSC); // Start conversion
  while (bit_is_set(ADCSRA, ADSC)); // measuring

  delay(2); // settling
  uint8_t low  = ADCL; // must read ADCL first - it then locks ADCH
  uint8_t high = ADCH; // unlocks both

  long result = (high << 8) | low;

  result = 1125300L / result; // Calculate Vcc (in mV); 1125300 = 1.1*1023*1000
  //  result = 2619000L / result; // Calculate Vcc for ProMicro??; 2619000 = 2.56*1023*1000

  return result; // Vcc in millivolts
 #endif
}

void readVoltages() {  // Reports battery voltage (or USB-Vcc volts) plus external ADCs
  //  B1: for battery voltage (or USB-Vcc volts)  B2: ADC_a,   B3 ADC_b
  // Volts are given as number of LEDs lit for whole 'V's, beep for 'period' and LEDs counted off for tenths of a volt. 
  // used with RP2040, reads out 3.3v level not battery, and requires 56K to GND and (A2+A3) connected
  long Vcc = 0;
  int V, d;
  float maxV=5.0, v;
  int Ain = 0;
  int fullScale=1024;

 #ifdef ProMicro
  Ain = ADC_a;
  fullScale=4096;
 #endif
 #ifdef RP2040
  maxV=3.3;
  //fullScale=4096;   // some libs return 10 bits & some 12 for ADC 0n RP2040 
 #endif

  while (!btn4) { // cycle till exit
    clearDisp();
    refreshWait(500);

    // btn1-3 will switch which analog line is read
    if (btn1) Ain = 0;
    if (btn2) Ain = ADC_a;
    if (btn3) Ain = ADC_b;
    
    if (Ain == 0) {
      Vcc = readVcc(); // supply voltage in milliVolts
#ifdef RP2040
      pinMode(ADC_a, INPUT_PULLUP);  // takes it high but w/ very high R
      pinMode(ADC_b, INPUT_PULLUP);  // these Pull-ups seem reversed, so do both
      //pinMode(29, OUTPUT);  // with 56k resistor between ADC_a & _b 
      //digitalWrite(29,LOW); // this will split inhalf the 3.3 operating voltage (not battery V)
      delay(10);
      Serial.println(analogRead(ADC_a));
      Vcc = 2* 1000 * maxV * analogRead(ADC_a) / fullScale;  // double the reading 
      Vcc = 3 * Vcc /4; // imperical fudge factor (set with known 3.3v)
 #endif
      v = Vcc / 1000.0;
      // print out the voltage (Vcc) value
      Serial.print("Vcc = "); Serial.print(Vcc / 1000); Serial.print("."); Serial.print(Vcc % 1000);
      // USB supplied Vcc should be > 4.5 volts; LiIon battery supplied should be <4.0
      // a Vcc reading of 4.0-4.5 indicated some kind of issue
      if (Vcc < 4300 && maxV==5.0) {  //... asume we are powered from the battery
        v = (Vcc + 250) / 1000.0; // +300 for the 0.25 volts across the 1N5817 diode
        Serial.print("    Battery ~= "); Serial.print(v);
      }
      Serial.println();
    } else {
      pinMode(ADC_a, INPUT); 
      pinMode(ADC_b, INPUT); 
      if (Ain == ADC_a) {
        v = maxV * analogRead(ADC_a) / fullScale;
        Serial.print("ADC-a voltage = "); Serial.println(v);
      } else {
        v = maxV * analogRead(ADC_b) / fullScale;
        Serial.print("ADC-b voltage = "); Serial.println(v);
      }
    }
    
    v=(v+0.05); // round up to 0.1 volt units
    V = int(v); // form V.d  V=whole Volts, d= 1/10ths
    d = int((v - V +0.05) * 10.0);
    if (V >= 1) {
      for (int i = 1; i <= V; i++) {
        brt[i] = true;
      }
    }
    refreshWait(350);
    beep(30);
    showLevel(d);

    DispScanWait(1000); // repeat every 1 second +
    if (Btn) buZZ(40); // ack you noted the btn press
  }
}

// =====================================================================
// ***************************  Group 6  *******************************
// ----------- Morse Code Practice & Exercise samples ------------------
// ----------- found in Morse_Code.h ------------------
// 

// =====================================================================
// ***************************  Group 7  *******************************
// --------------  Experimental / Under Development  -------------------
// ----------- found in Experimental.h ------------------

// =====================================================================
// ===========================  Group 0  ===============================
// -------------- Diagnostics Tools & Testing Utilities ----------------

void btnReporting() {  // Button - LED exercise & Reporting
  // ---------------------------------------------------------
  int icnt = 0; // iteration count, lsb~=1msec
  clearDisp();
  while (1) {
    refreshWait(1);
    icnt++;
    scanBtns();
    // ------------------- button reporting
    if (BtnChanged) {
      Serial.println(" ");
      if (btn1) Serial.print("[Btn1] "); else Serial.print("       ");
      if (btn2) Serial.print("[Btn2] "); else Serial.print("       ");
      if (btn3) Serial.print("[Btn3] "); else Serial.print("       ");
      if (btn4) Serial.print("[Btn4] "); else Serial.print("       ");
      Serial.print("  btnNum:"); Serial.print(btnNum);
      Serial.print("  btnState:"); Serial.print(btnState);
      if (btns) Serial.print("  btns!");
      if (BtnPressed) Serial.print("  BtnPressed!");

      // ------------------ LED reflection of button state
      lit[1] = lit[2] = btn1; // set Red LEDs
      lit[3] = lit[4] = btn2; // set Yel LEDs
      lit[5] = lit[6] = btn3; // set Blu LEDs
      lit[7] = lit[8] = btn4; // set Grn LEDs
      brightOne = 0;

      beep(15); // make a key press click, for up&down
      if (Btn) digitalWrite(AUDIO_OUT, HIGH);  // or will buZZ with the use of Piezo BUZZer rather than a sounder
      icnt = 0;
    } else if (icnt > 10000) { // if ignorned for 10 seconds
      icnt = 0;
      brightOne = 1;
      beep(15, 200);
      clearDisp();
    }
    if (ESC && brightOne == 0 && btn4 ) Serial.print(" [ESC] ");
    brightOne = (ESC) ? 8 : 0;
    if (ESC && btn1) break;    // only leave Btn Reporting if btn1 is also held.
  }
}

void Color_lights() {   // Exercise and test Color operations of . . . 
  // turns the 4 sets of LEDs On-Off, indivisually (exersises 'ledsAll()')
  int leds = 0;
  byte coLvl[5] = {0, 0, 0, 0, 0}; // level for each color LED set
  byte col_leds[] = {0, 1, 2, 4, 8};
  byte col_pat[] = {0, 0b00000011, 0b00001100, 0b00110000, 0b11000000};
  byte colNum;
  int baseFreq=700;

  Serial.println("Exercise and test operation of set_LEDs(), ledsAll(), showColor() & setColor() utils.");
  Beep(100);
  clearDisp();

  wait_BtnRelease(); // wait for no button
  blinkLed = 1;
  while (!catchBtn()) {
    ;    // TBD  is something missing here ?
  }
  blinkLed = 0;

  while (1) {
    if (BtnPressed) {
      colNum = btnNum;
      coLvl[colNum] = (coLvl[colNum] + 1) % 5;
      set_LEDs(col_pat[colNum], 0);  // clear old color set's levels, so not additive
      // update the lit state of each set of color LEDs
      switch (coLvl[colNum]) {
        case 1: set_LEDs(col_pat[colNum], 1); break;  // sets to dim level
        case 2:
          //ledsAll(col_leds[colNum]); // sets to mid lit level for 1-4 colors & clears 'lit[]' for others
          //showColor(colNum);         // shows only requested color (via 'lit[]') clears the rest of lit[]
          set_LEDs(col_pat[colNum], 2); // additively sets to mid lit level, requested color(s)
          break;
        case 3: set_LEDs(col_pat[colNum], 3); break;  // sets to HI lite level
        case 4:
          setColor(colNum); // overrides till cleared, does not relay on refreshDisp() resources
          wait_BtnRelease();
          setColor(0);  // clear setColor override, settings supported in refreshDisp will be seen again
          coLvl[colNum] = 0; // & reset lvl to 0
          break;
        default:  break;
      }
        // demo some sound effect, wide freq notes
        while (btnDown()) {
          Wave(baseFreq+100*btnNum, baseFreq+100*btnNum+125, 2, 100);
          if (ESC && btn1) return;
        }
    }

    // highlight any change with the onBrd Led
    digitalWrite(ledOnBrd, BtnChanged);
    if (BtnChanged) refreshWait(10);  // else ledOnBrd will be on for only 2 ms, sight unseen

    refreshWait(2);  // extra time extends ESC timeout period (1.5 -> 3 secs)
    scanBtns();
    if (ESC) return;
  }
}

void CursorGlowBeepTest() {  // facilitate evaluation of potential Cursor implementations
  byte code[5];
  byte i, s, e, loc = 1, cType = 1, audio = 0;
  byte litLevel = 0;
  byte dimItLoc = 0; //=2; // was for test viewing of cursors + dimIt

  Serial.println(" Move potential Cursors/Sprites accross a background 'display' field");
  Serial.println("   Cursors: Dim-Flash,Flash,Blink, dimIt, Dim, Lit & Bright LEDs");
  Serial.println("   Field:   Off, Dim, Lit, Bright, Arc, Glow LEDs  or levels 1-8 or 9-16");
  clearDisp();
  while (Btn) scanBtns(); // wait for btns to clear
  dimFlash = 1;

  glowEnable = true; // we here-in use glowing LEDs
  dimIt = dimItLoc;
  dim[3] = lit[4] = brt[5] = 1;
  arc[6] = glow[7] = 1;

  // handle btn presses
  while (!ESC) {
    if (BtnPressed) {
      if (btn1 && btn4) return;  // fast exit
      if (btn1Pressed) audio = audio % 4 + 1; // 1-4
      if (btn2Pressed) loc = (loc == 1) ?  nleds : loc - 1; // -- move left Allow 0:debugging
      if (btn3Pressed) loc = (loc % nleds) + 1; // ++ moves right
      if (btn4Pressed) cType = (cType % 7) + 1; // switch curror type

      if (BtnPressed && btn1 && btn2) {  // switches litLevels 1-8 & 9-16 for testing
        clearDisp();
        litLevel = (litLevel + 1) % 3;
        if (litLevel == 0) {
          dim[3] = lit[4] = brt[5] = 1;
          arc[6] = glow[7] = 1;
        } else if (litLevel == 1) {
          levelOn[1] = 1; levelOn[2] = 2; levelOn[3] = 3; levelOn[4] = 4;
          levelOn[5] = 5; levelOn[6] = 6; levelOn[7] = 7; levelOn[8] = 8;
        } else {
          levelOn[1] = 9; levelOn[2] = 10; levelOn[3] = 11; levelOn[4] = 12;
          levelOn[5] = 13; levelOn[6] = 14; levelOn[7] = 15; levelOn[8] = 16;
        }
      }

      // testing cursors over each of the possible LED states
      dimFlash = flashLed = blinkLed = dimLED = litLED = spriteOne = 0;
      if (cType == 1) dimFlash = loc;
      if (cType == 2) flashLed = loc;
      if (cType == 3) blinkLed = loc;
      if (cType == 4) dimIt = loc; else dimIt = dimItLoc; // works but subtle & not over blank cells, so can confusing
      //      if (cType==4) cType++;
      if (cType == 5) dimLED = loc;
      if (cType == 6) litLED = loc;
      if (cType == 7) spriteOne = loc;

      // test for audio interfering with display effects (check various cursors over the LED states)
      while (btnNum == 1 && !btns) {
        if (audio == 1) boop(50);
        if (audio == 2) beep(50);
        if (audio == 3) Beep(50);
        if (audio == 4) buZZ(50);
        refreshWait(200);
        scanBtns();
      }
    }
    refreshWait(1);
    scanBtns();
  }
}


void DebounceTest() { // user watches for lit LED skipping ahead of button press count
  unsigned int t;
  byte bb = 0;
  litLED = 1;
  while (!ESC) {
    scanBtns();
    if (BtnChanged) { // print time between state chgs
      t = millis() - t0;
      t0 = millis();
      Serial.print(" "); Serial.print(t);
      if ((!btns) && t < 20) { // count # of state time periods <20ms
        Serial.print(" _@"); Serial.print(++bb);
      }
      if (BtnPressed) litLED = (litLED % nleds) + 1; // move lit LED each time btn pressed
    }
    refreshWait(1);
  }
}


// ================== Local Support Utility Functions =================
// ====================================================================

//const int RedSet=4, YelSet=3, GrnSet=1;  // used by readySetGo() to cycle Red/Yellow/!GREEN
int readySetGo(int maxDelay) {  // -------  in support of Head-2-head Games:
  bool falseStarts = false;
  int goDelay;

  while (Btn) scanBtns();     // start with btns clear

  if (maxDelay != 0) falseStarts = true; // pentalize false starts
  setColor(RedSet); noBtnDelay(500);   // Red    - ready?
  setColor(0); noBtnDelay(200);
  setColor(YelSet); noBtnDelay(500);   // Yellow - Set

  goDelay = (maxDelay > 0) ? random(100, maxDelay) : 200;
  setColor(0); noBtnDelay(goDelay);

  if (Btn && falseStarts) return 1;
  if (ESC || btn3) {
    return 1;
  }

  setColor(GrnSet);     // ...                 // Green - GO!
  beep(25);  // the gun shot
  return 0;
  // calling code must do a 'clearDisp()' soon (to clear the setColor Grn)
}
